from TileType import *
from TDPUtilities import *
from TDSUtilities import *

logic1 = "1"
logic0 = "0"
invalid = "X"

MSB = "m"
REG = "r"
LSB = "l"
MSBLSB = "lm"
LSBMSB = "lm"

t0delimiter = ":"
t1delimiter = "|"
t2delimiter = "."

borrowSet = ["nb", "b"]
prop = "prop"
resetSet = ["sleep", "wake"]
passMSB = "MSB"
s2 = "s2"
Seed = "Seed"
reset = "Reset"
zero = "Zero"
nonZeroSignal = "Continue"

primaryDirection = N
primaryDirectionReverse = S
secondaryDirection = E
secondaryDirectionReverse = W
tertiaryDirection = U
tertiaryDirectionReverse = D

def mem(d,c=""):
	return t2delimiter.join([d,c])

def data(d,c=""):
	return t2delimiter.join([d,c])

def join(data, mem):
	return t1delimiter.join([data, mem])

def joinList(dataList, memList):
	result = []
	for mem in memList:
		for data in dataList:
			result.append(join(data, mem))
	return result

secondaryInstructions = {
	data(logic1, MSB): [s2] + [passMSB] + borrowSet,
	data(logic0, MSB): [s2],
	data(logic1, REG): borrowSet + [prop],
	data(logic0, REG): borrowSet + [prop],
	data(logic1, LSB): [prop] + [reset],
	data(logic0, LSB): [s2] + [reset],
	data(logic1, LSBMSB) : [s2] + [passMSB],
	data(logic0, LSBMSB) : [],
	invalid : resetSet + [reset]
	}

validMSBmem = [mem(logic1, MSB)]
validREGmem = [mem(logic1, REG), mem(logic0, REG)]
validLSBmem = [mem(logic1, LSB), mem(logic0, LSB)]
validLSBMSBmem = [mem(logic1, LSBMSB)]
allValidmem = validMSBmem + validREGmem + validLSBmem

validMSBdata = [data(logic1, MSB), data(logic0, MSB)]
validREGdata = [data(logic1, REG), data(logic0, REG)]
validLSBdata = [data(logic1, LSB), data(logic0, LSB)]
validLSBMSBdata = [data(logic1, LSBMSB), data(logic0, LSBMSB)]

validMSBcombo = joinList(validMSBdata, validMSBmem)
validREGcombo = joinList(validMSBdata, validREGmem) + joinList(validREGdata, validREGmem)
validLSBcombo = joinList(validLSBMSBdata, validLSBmem) + joinList(validLSBdata, validLSBmem)
invalidCombo = joinList([invalid], validMSBmem) + joinList([invalid], validREGmem)

validMaster = validMSBcombo + validREGcombo + validLSBcombo + invalidCombo

def getTileDataMem(tileName):
	data1 = tileName.split(t0delimiter)
	for d1 in data1:
		try:
			side, role = d1.split(t2delimiter)
		except:
			side = ""
			role = ""
		dict1[side] = role
		try:
			return dict1["pR"]
		except:
			return ""




	

def baseRuleSet(primaryInput, secondaryInput, counterLayer=0, verticalSignal=0):
	result = {
	"pReverse":(1, primaryInput), "p":(0, ""),
	"sReverse":(0, ""), "s":(0, ""),
	"tReverse":(0, ""), "t":(0, "")
	}
	if t1delimiter in primaryInput:
		data, mem = primaryInput.split(t1delimiter)
		if invalid not in data:
			dbit, dMRL = data.split(t2delimiter)
		mbit, mMRL = mem.split(t2delimiter)
		if invalid in data:
			result["sReverse"] = (1, secondaryInput)
			if secondaryInput == resetSet[0]:
				if mMRL != MSB:
					result["s"] = (1, secondaryInput)
				result["p"] = (1, primaryInput)
			elif secondaryInput == resetSet[1]:
				if mMRL == LSB:
					result["p"] = (2, join(mem, mem))
				else:
					result["p"] = (1, join(mem, mem))
				if mMRL != MSB:
					result["s"] = (1, secondaryInput)
			elif secondaryInput == reset:
				result["pReverse"] = (1, join(mem, mem))
				result["p"] = (1, join(mem, mem))
				if mMRL != MSB:
					result["s"] = (1, reset)
		elif secondaryInput == s2:
			result["pReverse"] = (2, primaryInput)
			if dbit == logic0 and dMRL == MSB:
				result["p"] = (1, join(invalid, mem))
				result["sReverse"] = (1, passMSB)
				if mMRL != MSB:
					result["s"] = (1, resetSet[0])
			elif dbit == logic1 and dMRL == MSB:
				result["p"] = (1, primaryInput)
				result["sReverse"] = (1, prop)
				if mMRL != MSB:
					result["s"] = (1, resetSet[0])
			elif dbit == logic0 and dMRL == LSB:
				result["p"] = (1, join(t2delimiter.join([logic1, dMRL]), mem))
				result["s"] = (1, borrowSet[1])
			elif dbit == logic1 and dMRL == LSBMSB:
				result["p"] = (1, join(t2delimiter.join([logic0, dMRL]), mem))
				result["s"] = (1, resetSet[0])
			elif dbit == logic0 and dMRL == LSBMSB:
				result["p"] = (2, t2delimiter.join([join(mem, mem),reset]))
				result["s"] = (1, resetSet[1])
			elif dbit == logic1 and dMRL == LSB:
				result["p"] = (2, join(mem, mem))
				result["s"] = (1, prop)
				result["sReverse"] = (1, "RESET")
		elif secondaryInput == reset:
			result["pReverse"] = (2, t2delimiter.join([primaryInput,reset]))
			if dbit == logic1 and dMRL == LSB:
				result["p"] = (2, join(t2delimiter.join([logic0, dMRL]), mem))
				result["s"] = (1, reset)
				result["sReverse"] = (1, "RESET")
			elif dbit == logic0 and dMRL == LSB:
				result["p"] = (1, join(t2delimiter.join([logic1, dMRL]), mem))
				result["s"] = (1, borrowSet[1])
		elif secondaryInput == passMSB:
			if mMRL != LSB:
				result["pReverse"] = (1, join(t2delimiter.join([logic1, REG]), mem))
				result["s"] = (1, passMSB)
				result["p"] = (1, join(t2delimiter.join([logic1, MSB]), mem))
				result["sReverse"] = (1, prop)
			else:
				result["pReverse"] = (1, join(t2delimiter.join([logic1, LSB]), mem))
				result["s"] = (1, passMSB)
				result["p"] = (1, join(t2delimiter.join([logic0, LSBMSB]), mem))
		else:
			if secondaryInput in borrowSet:
				result["sReverse"] = (1, secondaryInput)
				if secondaryInput == borrowSet[1]:
					if dbit == logic0:
						result["p"] = (1, join(t2delimiter.join([logic1, dMRL]), mem))
						result["s"] = (1, secondaryInput)
					else:
						result["p"] = (1, join(t2delimiter.join([logic0, dMRL]), mem))
						result["s"] = (1, borrowSet[0])
						if dMRL == MSB:
							result["p"] = (2, join(t2delimiter.join([logic0, dMRL]), mem))
							result["s"] = (1, resetSet[0])
				else:
					if dMRL == MSB:
						result["p"] = (2, primaryInput)
						result["s"] = (1, resetSet[0])
					else:
						result["p"] = (1, primaryInput)
						result["s"] = (1, secondaryInput)
			elif secondaryInput == prop:
				result["s"] = (1, secondaryInput)
				if dMRL == LSB:
					result["p"] = (2, join(t2delimiter.join([logic0, LSB]), mem))
				else:
					result["p"] = (1, primaryInput)
					result["sReverse"] = (1, secondaryInput)
				if dMRL == MSB:
					result["sReverse"] = (1, resetSet[0])
				
	else:
		print "Warning: {} does not contain {}".format(primaryInput, t1delimiter)
		return {}
	if LSB in mMRL:
		result["t"] = (1, nonZeroSignal + str(counterLayer))
		if verticalSignal:
			result["t"] = (2, nonZeroSignal + str(counterLayer))
		result["pReverse"] = (result["pReverse"][0], "!".join([result["pReverse"][1], "layer{}".format(counterLayer)]))
		result["p"] = (result["p"][0], "!".join([result["p"][1], "layer{}".format(counterLayer)]))
		result["sReverse"] = (1, "LSB")
	return result

def makeTileFromRuleSet(ruleSet, counterLayer=0):
	
	nameList = ["" for i in range(6+1)]
	t = TileType(True)
	for side, rule in ruleSet.items():

		strength = rule[0]
		if strength == 0:
			continue
		try:
			label = rule[1].replace(t2delimiter,"")
		except:
			print "Unexpected rule: {}\t{}".format(side, rule)
		glue = Glue(strength, label)
		if side == "p":
			t.sides[primaryDirection] = glue
			nameList[5] = t2delimiter.join(["p", label])
		elif side == "pReverse":
			t.sides[primaryDirectionReverse] = glue
			nameList[0] = t2delimiter.join(["pR", label])
			data, mem = label.split(t1delimiter)
			if logic1 in data:
				t.color = "Blue"
			elif invalid in data:
				t.color = "Red"
		elif side == "s":
			t.sides[secondaryDirection] = glue
			nameList[3] = t2delimiter.join(["s", label])
		elif side == "sReverse":
			t.sides[secondaryDirectionReverse] = glue
			nameList[1] = t2delimiter.join(["sR", label])
		elif side == "t":
			t.sides[tertiaryDirection] = glue
			nameList[4] = t2delimiter.join(["t", label])
		elif side == "tReverse":
			t.sides[tertiaryDirectionReverse] = glue
			nameList[2] = t2delimiter.join(["tR", label])
	#if t.sides[primaryDirection].strength == 2:
	#	t.color = "Cyan"
	
	t.name = t0delimiter.join(nameList)

	return t

def makeSeedTiles(tilelist=[]):
	seedTileList = []
	seedNameList = []
	for tile in tilelist:
		for mem in allValidmem:
			tileData = join(mem, mem).replace(t2delimiter,"")
			if tileData in tile.sides[primaryDirectionReverse].label:
				t = TileType(True)
				if MSB in mem:
					variant = tile.sides[secondaryDirectionReverse].label
					if tile.sides[secondaryDirectionReverse].label not in borrowSet + [prop]:
						continue
				elif REG in mem:
					variant = tile.sides[secondaryDirectionReverse].label
					
					if tile.sides[secondaryDirection].label not in borrowSet + [prop]:
						continue
				elif LSB in mem:
					variant = tile.sides[primaryDirectionReverse].label.split("!")[-1]
					if tile.sides[secondaryDirection].label == passMSB:
						continue
				t.name = t0delimiter.join([Seed, tileData, variant])
				t.sides[primaryDirectionReverse] = Glue(2, t.name)
				t.sides[primaryDirection] = tile.sides[primaryDirection].Copy()
				t.sides[tertiaryDirection] = tile.sides[tertiaryDirection].Copy()
				t.color = tile.color
				t.color = tile.color
				
				if t.name not in seedNameList:
					seedNameList.append(t.name)
					seedTileList.append(t)

	return seedTileList

def generateSignalTiles(layer, verticalSignal, signal):
	result = []
	for i in range(verticalSignal):
		i = i + 1
		t = TileType(True)
		t.name = "Layer{}|Signal{}|{}/{}".format(layer, signal, i, verticalSignal)
		t.sides[tertiaryDirectionReverse] = Glue(2, "Layer{}|{}|{}/{}".format(layer, signal, i, verticalSignal))
		t.sides[tertiaryDirection] = Glue(2, "Layer{}|{}|{}/{}".format(layer, signal, i+1, verticalSignal))
		if i == verticalSignal:
			t.sides[tertiaryDirection] = Glue(1, signal)
		if i == 1:
			t.sides[tertiaryDirectionReverse] = Glue(2, signal)
		if nonZeroSignal not in signal:
			t.color = "Red"

		result.append(t)

	return result


def designZeroTiles(layer, verticalSignal=0):
		baseZeroTiles = []
		allZeroTiles = []
		signalTiles = []
		for mem in validLSBmem:
			mem = mem.replace(t2delimiter, "")
			t = TileType(True)
			t.name = ":".join(["ZERO", str(layer), mem])
			t.sides[primaryDirectionReverse] = Glue(1, logic0 + LSBMSB + t1delimiter + mem + "!" + "layer" + str(layer))
			t.sides[primaryDirection] = Glue(2, join(mem, mem)+reset+"!"+"layer"+str(layer))
			t.sides[secondaryDirection] = Glue(1, resetSet[1])
			t.sides[secondaryDirectionReverse] = Glue(1, "ZERO")
			baseZeroTiles.append(t)
		if verticalSignal:
			signalTiles += generateSignalTiles(layer, verticalSignal, nonZeroSignal+str(layer))

		if layer == 0:
			for tile in baseZeroTiles:
				t = tile.Copy()
				signal = "".join(["l{}".format(i) for i in [0]])
				t.name = signal
				t.sides[tertiaryDirection] = Glue(1, signal)
				if verticalSignal:
					t.sides[tertiaryDirection].strength = 2
					signalTiles += generateSignalTiles(layer, verticalSignal, signal)
				allZeroTiles.append(t)
		elif layer == 1:
			for tile in baseZeroTiles:

				signal = "".join(["l{}".format(i) for i in [0,1]])
				signalIn = "".join(["l{}".format(i) for i in [0]])
				t = tile.Copy()
				t.name = ":".join([tile.name, signalIn])
				t.sides[tertiaryDirectionReverse] = Glue(1, signalIn)
				t.sides[tertiaryDirection] = Glue(1, signal)
				if verticalSignal:
					t.sides[tertiaryDirection].strength = 2
					signalTiles += generateSignalTiles(layer, verticalSignal, signal)
				t.sides[primaryDirection] = Glue()
				t.sides[secondaryDirection] = Glue()
				allZeroTiles.append(t)


				signal = "".join(["l{}".format(i) for i in [1]])
				signalIn = nonZeroSignal  + str(layer - 1)
				t = tile.Copy()
				t.name = ":".join([tile.name, nonZeroSignal])
				t.sides[tertiaryDirectionReverse] = Glue(1, signalIn)
				t.sides[tertiaryDirection] = Glue(1, signal)
				if verticalSignal:
					t.sides[tertiaryDirection].strength = 2
					signalTiles += generateSignalTiles(layer, verticalSignal, signal)
				allZeroTiles.append(t)
		elif layer == 2:
			for tile in baseZeroTiles:

				signal = "".join(["l{}".format(i) for i in [2]])
				signalIn = nonZeroSignal + str(layer - 1)
				t = tile.Copy()
				t.name = ":".join([tile.name, nonZeroSignal])
				t.sides[tertiaryDirectionReverse] = Glue(1, signalIn)
				t.sides[tertiaryDirection] = Glue(1, signal)
				if verticalSignal:
					t.sides[tertiaryDirection].strength = 2
					signalTiles += generateSignalTiles(layer, verticalSignal, signal)
				allZeroTiles.append(t)

				signal = "".join(["l{}".format(i) for i in [1,2]])
				signalIn = "".join(["l{}".format(i) for i in [1]])
				t = tile.Copy()
				t.name = ":".join([tile.name, signalIn])
				t.sides[tertiaryDirectionReverse] = Glue(1, signalIn)
				t.sides[tertiaryDirection] = Glue(1, signal)
				if verticalSignal:
					t.sides[tertiaryDirection].strength = 2
					signalTiles += generateSignalTiles(layer, verticalSignal, signal)
				allZeroTiles.append(t)

		for tile in allZeroTiles:
			tile.color = "Red"

		return allZeroTiles + signalTiles

def baseTileSet(layer=0, verticalSignal=0):
	tileSet = []
	dataMemCombinations = validMaster
	for dataMem in dataMemCombinations:
		data, mem = dataMem.split(t1delimiter)
		for secondaryInstruction in secondaryInstructions[data]:
			rule = baseRuleSet(dataMem, secondaryInstruction, counterLayer=layer, verticalSignal=verticalSignal)
			t = makeTileFromRuleSet(rule, counterLayer=layer)
			if not t.IsEmpty():
				tileSet.append(t)
	tileSet += makeSeedTiles(tileSet)
	return tileSet



class PeriodicCounter:
	tdp = TDP("{}.tdp".format("default"))
	tds = TDS("{}.tds".format("default"))
	tdp.tdsFile = tds.name
	layer = 0
	seedTiles = []

	def __init__(self, layer=0, verticalSignal=0):
		self.tds = TDS("PC_l{}.tds".format(layer))
		self.tdp = TDP("PC_l{}.tdp".format(layer))
		self.tdp.tdsFile = self.tds.name
		self.layer = layer
		self.tds.AddTiles(baseTileSet(self.layer, verticalSignal))
		self.tds.AddTiles(designZeroTiles(self.layer, verticalSignal))
		self.seedTiles = [t for t in self.tds.tileList if Seed in t.name]
		#self.interactionAliases = defaultInteractions(layer)

	def seed(self, n, location=[0,0,0]):
		binary = bin(n).replace("0b","")
		binary = binary[::-1]
		LSBlocation = 0
		MSBlocation = len(binary) - 1
		variant = ""

		self.tdp.TileAndLocationList = []
		borrow = True
		even = (binary[LSBlocation] == "0")
		for i, bit in enumerate(binary):
			if i == MSBlocation:
				mem = logic1 + MSB
				tileData = join(mem, mem)
				if even:
					if binary.count("1") > 1:
						variant = borrowSet[0]
					else:
						variant = borrowSet[1]
				else:
					variant = prop
			elif i == LSBlocation:
				if even:
					mem = logic0 + LSB
				if not even:
					mem = logic1 + LSB
				tileData = join(mem, mem)
				variant = "layer{}".format(self.layer)
			else:
				if even:
					if borrow:
						variant = borrowSet[1]
					else:
						variant = borrowSet[0]
				else:
					variant = prop
				if binary[i] == "0":
					mem = logic0 + REG
				if binary[i] == "1":
					mem = logic1 + REG
				tileData = join(mem, mem)
			self.tdp.AddTile(t0delimiter.join([Seed, tileData, variant]), [location[0]+i, location[1]+0, location[2]+0])
			if bit == "1":
				borrow = False

		return self.tdp

def test():
	#print defaultInteractions(2).keys()
	tdpMaster = TDP("PC_Master.tdp")
	tdsMaster = TDS("PC_Master.tds")
	tdpMaster.tdsFile = tdsMaster.name

	PC_0 = PeriodicCounter(0)
	PC_1 = PeriodicCounter(1)
	tdpMaster.Merge(PC_0.seed(80, [0,0,0]))
	tdpMaster.Merge(PC_1.seed(19, [0,0,1]))

	print len(PC_0.tds.tileList)
	print len(PC_1.tds.tileList)
	tdsMaster.Merge(PC_0.tds)
	print len(tdsMaster.tileList)
	tdsMaster.Merge(PC_1.tds)
	print len(tdsMaster.tileList)

	tdpMaster.Write()
	tdsMaster.Write()

	tdsSeed = TDS("PC_Seed.tds")
	for tile in tdsMaster.tileList:
		if Seed in tile.name:
			tdsSeed.AddTile(tile)

	tdsSeed.Write