# improved TDP and TDS utilities

from TileType import *
import copy
import os

class Assembly:
	def __init__(self, name="", tdsFile="", temperature=2, tiles={}):
		if '.tdp' not in name:
			name += '.tdp'
		self.name = name
		self.tiles = {}
		self.temperature = temperature
		if '.tds' not in tdsFile:
			tdsFile += '.tds'
		self.tdsFile = tdsFile

	def Write(self, directory=''):
		if directory:
			if not os.path.exists(directory):
				os.makedirs(directory)
			filename = os.path.join(directory, self.name)
		else:
			filename = self.name
		with open(filename, 'w') as outFile:
			outFile.write(self.Output())
		print "Wrote {} tiles to {}".format(len(self.tiles), filename)

	def Output(self):
		ret = self.tdsFile + '\n'
		ret += 'Temperature=' + str(self.temperature) + '\n'
		ret += self.OutputWithoutHeader()
		return ret

	def OutputWithoutHeader(self):
		ret = ''
		for loc, name in self.tiles.iteritems():
			ret += name + ' ' + ' '.join([str(x) for x in loc]) + '\n'
		return ret

	def AddTile(self, name, loc):
		self.tiles[tuple(loc)] = name

	def Merge(self, otherTDP):
		for loc, name in otherTDP.tiles.iteritems():
			self.AddTile(name, loc)

def ParseAssembly(tdpFileName):
	tdp = Assembly(tdpFileName)
	if ".tdp" not in tdpFileName:
		print "ERROR, INVALID SUFFIX FOR " + tdpFileName + "\n"
		print "Suffix should be .tdp\n"
		return None
	with open(tdpFileName, "r") as tdpFile:
		content = tdpFile.readlines()
		for line in content:
			line = line.strip("\n")
			if ".tds" in line:
				tdp.tdsFile = line
				continue
			elif "Temperature=" in line:
				tdp.temperature = int(line.replace("Temperature=",""))
				continue
			elif line == "":
				continue
			else:
				data = line.split(" ", 1)
				name = data[0]
				loc = tuple([int(x) for x in data[1].split()])
				tdp.AddTile(name, loc)
	return tdp
