from TileType import *
import os

class TileSet:
    def __init__(self, name, tileList=None, tiles={}):
        if '.tds' not in name:
            name += '.tds'
        self.name = name
        self.tiles = tiles
        if tileList is not None:
            for tile in tileList:
                self.tiles[tile.name] = tile

    def Write(self, directory=''):
        if directory:
            if not os.path.exists(directory):
                os.makedirs(directory)
            filename = os.path.join(directory, self.name)
        else:
            filename = self.name
        with open(filename, 'w') as outFile:
            for name, tile in self.tiles.iteritems():
                outFile.write(tile.Output() + '\n')
        print 'Wrote {} tiles to {}'.format(str(len(self.tiles)), filename)

    def Merge(self, other):
        for name, tile in other.tiles.iteritems():
            if name not in self.tiles:
                self.AddTile(tile)

    def AddTiles(self, tileList):
        for tile in tileList:
            if tile.name not in self.tiles:
                self.AddTile(tile)

    def AddTile(self, tile):
        tile = tile.Copy()
        if tile.name in self.tiles:
            print "A tile with name \"{}\" already exists in {}".format(tile.name, self.name)
            oldname = tile.name
            i = 1
            while '{}({})'.format(tile.name, str(i)) in self.tiles:
                i += 1
            tile.name = '{}({})'.format(tile.name, str(i))
            print 'Name: \"{}\" was changed to \"{}\"'.format(oldname, tile.name)
        self.tiles[tile.name] = tile

def ParseTileSet(tdsFileName):
    tdsFileName = tdsFileName.strip()

    if ".tds" not in tdsFileName:
        print "Invalid File Name: Lacking .tds suffix"
        return None

    tiles = {}

    with open(tdsFileName, 'Ur') as inFile:
        content = inFile.readlines()
        for line in content:

            #Pre-Processing
            line = line.strip()
            lineSplit = line.split(" ")
            lineInstruction = lineSplit[0]
            if len(lineSplit) > 1:
                lineData = lineSplit[1]

            #Parse to Tile
            if "TILENAME" in lineInstruction:
                t = TileType(True)
                t.name = lineData
                continue
            elif "LABEL" in lineInstruction and lineInstruction not in [dirName + "LABEL" for dirName in DirNameArray]:
                t.label = lineData
                continue
            elif "TILECOLOR" in lineInstruction:
                t.color = " ".join(lineSplit[1:])
                continue
            elif "CREATE" in lineInstruction:
                tiles[t.name] = t
                continue
            else:
                for dirNum, dirName in enumerate(DirNameArray):
                    if dirName.upper() in line:
                        if "BIND" in line:
                            t.sides[dirNum].strength = int(lineData)
                            break
                        if "LABEL" in line:
                            t.sides[dirNum].label = lineData
                            break
        tds = TileSet(tdsFileName, tiles=tiles)
    return tds
