from TileType import *
import copy
import os

class TDS:
	name = ""
	tileList = []
	tileDictionary = {}

	def __init__(self, name, tileList = [], tileDictionary = {}):
		if ".tds" not in name:
			name += ".tds"
		self.name = name
		if tileList == []:
			self.tileList = list()
		else:
			self.tileList = tileList
		if tileDictionary == {}:
			self.tileDictionary = dict()
		else:
			self.tileDictionary = tileDictionary

	def Write(self, directory=""):
		if directory:
			if not os.path.exists(directory):
				os.makedirs(directory)
			filename = os.path.join(directory, self.name)
		else:
			filename = self.name
		with open(filename, "w") as outFile:
			for tile in self.tileList:
				outFile.write(tile.Output())
				outFile.write("\n")
		print "Wrote " + str(len(self.tileList)) + " tiles to " + filename

	def Merge(self, otherTDS):
		for tile in otherTDS.tileList:
			if not self.tileDictionary.get(tile.name):
				self.AddTile(tile)
				#self.tileList += otherTDS.tileList
				#self.tileDictionary.update(otherTDS.tileDictionary)

	def AddTiles(self, tileList):
		for tile in tileList:
			if not self.tileDictionary.get(tile.name):
				self.AddTile(tile)

	def AddTile(self, tile):
		if self.tileDictionary.get(tile.name):
			print "A tile with name \"" + tile.name + "\" already exists in " + self.name + "\n"
			i = 1
			while self.tileDictionary.get(tile.name + "(" + str(i) + ")") != None:
				i += 1
			print "Name: \"" + tile.name + "\" was changed to \"" + tile.name + "(" + str(i) + ")\"\n"
			tile.name = tile.name + "(" + str(i) + ")" 
		self.tileDictionary[tile.name] = copy.deepcopy(tile)
		self.tileList.append(copy.deepcopy(tile))	

def parseTDS(tdsFileName):
	tdsFileName = tdsFileName.strip()
	tds = TDS(tdsFileName)

	if ".tds" not in tdsFileName:
		print "Invalid File Name: Lacking .tds suffix"
		return None

	l = []
	dictionary = {}

	with open(tdsFileName, 'Ur') as inFile:
		content = inFile.readlines()
		for line in content:

			#Pre-Processing
			line = line.strip()
			lineSplit = line.split(" ")
			lineInstruction = lineSplit[0]
			if len(lineSplit) > 1:
				lineData = lineSplit[1]

			#Parse to Tile
			if "TILENAME" in lineInstruction:
				t = TileType(True)
				t.name = lineData
				continue
			elif "LABEL" in lineInstruction and lineInstruction not in [dirName + "LABEL" for dirName in DirNameArray]:
				t.label = lineData
				continue
			elif "TILECOLOR" in lineInstruction:
				t.color = " ".join(lineSplit[1:])
				continue
			elif "CREATE" in lineInstruction:
				l.append(t)
				dictionary[t.name] = t
				continue
			else:
				for dirNum, dirName in enumerate(DirNameArray):
					if dirName.upper() in line:
						if "BIND" in line:
							t.sides[dirNum].strength = int(lineData)
							break
						if "LABEL" in line:
							t.sides[dirNum].label = lineData
							break
		tds.tileList = l
		tds.tileDictionary = dictionary
	return tds

def parseTDSList(tdsFileList):
	tdsList = []
	for tdsFileName in tdsFileList:
		tds = TDS(tdsFileName)
		tds.parseTilesFromTDS(tdsFileName)
		tdsList.append(tds)
	return tdsList