N = North = 0
E = East = 1
S = South = 2
W = West = 3
U = Up = 4
D = Down = 5

DirAbbrevArray = ("N", "E", "S", "W", "U", "D")
DirNameArray = ("NORTH", "EAST", "SOUTH", "WEST", "UP", "DOWN")
OppositeDir = {N:S, S:N, E:W, W:E, U:D, D:U,
    "N":"S", "E":"W", "S":"N", "W":"E", "U":"D", "D":"U"}

class Glue:
    """A representation of a glue on the side of a tile, including strength and lable"""
    strength = 0
    label = ""

    def __init__(self, strength = 0, label = ""):
        self.strength = strength
        self.label = label

    def __eq__(self, other):
        if (isinstance(other, Glue)):
            return (self.strength == other.strength) and (self.label == other.label)
        return False

    def Copy(self):
        return Glue(self.strength, self.label)

    def Contains(self, string):
        return (self.label.find(string) > -1)

    def Print(self, d):
        print(DirNameArray[d] + "BIND " + str(self.strength))
        print(DirNameArray[d] + "LABEL " + self.label)

    def Output(self, d):
        ret  = DirNameArray[d] + "BIND " + str(self.strength) + "\n"
        ret += DirNameArray[d] + "LABEL " + self.label + "\n"
        return ret

class TileType:
    """A representation of a tile type for use in the ISU TAS simulator.
       Every tile must have a unique name and a unique set of 4 glues. If
       a name is not explicitly provided, one will be formed automatcally
       as a concatenation of the glues values of each side."""

    name = ""
    label = ""
    color = ""
    sides = {}
    dims  = 2

    def __init__(self, is3D=False):
        if is3D == False:
            self.dims = 2
            self.sides = {N:Glue(),E:Glue(),S:Glue(),W:Glue()}
        else:
            self.dims = 3
            self.sides = {N:Glue(),E:Glue(),S:Glue(),W:Glue(),U:Glue(),D:Glue()}

    def Copy(self):
        if self.dims == 2:
            t = TileType()
        else:
            t = TileType(True)
        t.sides[N] = self.sides[N].Copy()
        t.sides[E] = self.sides[E].Copy()
        t.sides[S] = self.sides[S].Copy()
        t.sides[W] = self.sides[W].Copy()
        if self.dims == 3:
            t.sides[U] = self.sides[U].Copy()
            t.sides[D] = self.sides[D].Copy()
        t.name = self.name
        t.label = self.label
        t.color = self.color
        return t

    def Print(self):
        if self.IsEmpty():
            return
        if self.name == "":
            self.CreateName()
        print("TILENAME " + self.name)
        print("LABEL " + self.label)
        for i,d in self.sides.items():
            self.sides[i].Print(i)
        if len(self.color) > 0:
            print("TILECOLOR " + self.color)
        print("CREATE\n")

    def Output(self):
        if self.IsEmpty():
            return ""
        if self.name == "":
            self.CreateName()
        ret  = "TILENAME " + self.name + "\n"
        ret += "LABEL " + self.label + "\n"
        for i,d in self.sides.items():
            ret += self.sides[i].Output(i)
        if len(self.color) > 0:
            ret += "TILECOLOR " + self.color + "\n"
        ret += "CREATE\n"
        return ret

    def SetInputSides(self, inputSet):
        for k in inputSet:
            self.sides[k] = Glue(inputSet[k].strength, inputSet[k].label)

    def IsEmpty(self):
        for i in range(self.dims*2):
            if not self.sides[i].strength == 0:
                return False
        return True

    def GetName(self):
        if len(self.name) == 0:
            CreateName(self)
        return self.name

    def CreateName(self):
        for i in range(self.dims*2):
            self.name = self.name + DirAbbrevArray[i] + str(self.sides[i].strength) + self.sides[i].label
