from TileType import *
from TDSUtilities import *
from TDPUtilities import *
from genomeUtilities import *
from Orientation import *
from LocationUtilities import *

from sys import argv

import copy

# TDPdir = "./TDP/"
# TDSdir = "./TDS/"

tdsOut = TDS("CircularLatch.tds")
tdpOut = TDP("CircularLatch.tdp", tdsOut.name)
tdsSeed = TDS("CircularLatchSeed.tds")
tdsInData = parseTDS("./Components/3D-alldirs.tds")
tdsInInit = parseTDS("./Components/datapath_adder/Demo/DatapathStart.tds")

limitedTest = False
includeDataInSeed = False

intersectionOffset = 3
signalOffset = 2

################################################ VARIABLE SET UP ###########################################
baseTiles = []
baseTileNames = ["N","E","S","W","U","D","Check","TD"]
baseTileColors = {k:"Green" for k in baseTileNames}
baseTileColors["Check"] = "Yellow"
baseTileColors["TD"] = "Magenta"
for tile in tdsInData.tileList:
	if "Start" in tile.name:
		if "DN" in tile.name:
			baseTileNames.append(tile.name)
			baseTiles.append(tile)
			baseTileColors[tile.name] = tile.color

initTileNames = []
initTileNamesActive = []
initTiles = []
initTileColors = {}
for tile in tdsInInit.tileList:
	initTileNamesActive.append(tile.name)
	if tile.name not in baseTileNames:
		initTileNames.append(tile.name)
		initTiles.append(tile)
		initTileColors[tile.name] = tile.color

############################################# GROUPING ####################################################

instructionGroups = InstructionGroups(40, signalOffset)
def getGroup(travelDir):
	return instructionGroups[travelDir]

###################################### TDS #####################################
groups = set([k for k in instructionGroups.keys() if "Group" in k])

allRoles = list(groups) + singletonTiles + baseTileNames + initTileNames + validRoles

def roleColor(role):
	if "|" in role:
		return roleColor(DecodeNCPFormat(role)["Curr"])
	if role == "Equal":
		return "Green"
	if role == "NotEqual":
		return "Red"
	if role in validRoles:
		if role == "Prop":
			return "Blue"
		else:
			return "Red"
	if role in singletonTiles:
		if "Interface" in role:
			return "Green"
		else:
			return "Magenta"
	if role in list(groups):
		return "Magenta"
	if role in baseTileNames:
		if role == "TileDelimiter":
			return "Magenta"
		elif role == "SideDelimiter":
			return "Green"
		else:
			return baseTileColors[role]
	elif "X|" in role:
		return ""
	elif role in list(groups):
		return "Magenta"
	else:
		return "Gray"

def growthActivatePropGlue(travelDir):
	return Glue(1, ":".join(["PropGrowthActivation", travelDir]))

def growthActivateHelperGlue(travelDir):
	return Glue(1, ":".join(["GrowthActivationHelper", travelDir])) 

def growthActivateSwitchHelperGlue():
	return Glue(1, "GrowthActivationSwitchHelper")

def growthActivateStartGlue(travelDir):
	return Glue(1, ":".join(["GrowthActivationStart", travelDir]))

def growthActivateBandJumpGlue(travelDir):
	return Glue(1, ":".join(["GrowthActivationBandJump", travelDir]))

def growthActivatePreJumpGlue(travelDir):
	return Glue(1, ":".join(["GrowthActivationPreJump", travelDir]))

def growthActivatePostJumpGlue(travelDir):
	return Glue(1, ":".join(["GrowthActivationPostJump", travelDir]))

def generateTurnTiles():
	latch = False
	for travelDir in DirMods:
			
		dirs = PSTdir[travelDir]
		primaryDirection = DirAbbrevArray.index(dirs[0])
		secondaryDirection = DirAbbrevArray.index(dirs[1])
		tertiaryDirection = DirAbbrevArray.index(dirs[2])
		primaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[0]])
		secondaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[1]])
		tertiaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[2]])

		TurnFromStart = True
		if travelDir[0] in majorDirs:
			turnFromStart = True
		else:
			turnFromStart = False

		nonSignalingDirection = 0
		signalingDirection = 0

		if(travelDir in band0xband1 + band1xband2 + band0xband2):
			signalingDirection = tertiaryDirection
			nonSignalingDirection = tertiaryDirectionReverse
		else:
			signalingDirection = tertiaryDirectionReverse
			nonSignalingDirection = tertiaryDirection
 
		"""face = travelDir[1]
		if face in ["U", "D"]:
			outDir = DirAbbrevArray.index(face)
			inDir = DirAbbrevArray.index(oppositeByDir[face])
		elif face in ["N", "S"]:
			outDir = DirAbbrevArray.index(oppositeByDir[face])
			inDir = DirAbbrevArray.index(face)
		elif face in ["E", "W"]:
			outDir = DirAbbrevArray.index(oppositeByDir[face])
			inDir = DirAbbrevArray.index(face)

		if travelDir in band0 or travelDir in band1xband2:
			signalingDirection = outDir
			nonSignalingDirection = inDir
		else:
			signalingDirection = inDir
			nonSignalingDirection = outDir"""
 
		oppositeDir = oppositeByDirMod[travelDir]

		if turnFromStart:

			tileClass = "PreTurn0"
			nextClass = "PreTurnKeyRow"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				if turnTest:
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
				if instructionTest:
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				t.sides[nonSignalingDirection] = Glue(2, ":".join([role, "C-Latch", travelDir]))
				if(role == travelDir):
					t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
				t.color = roleColor(role)
				tdsOut.AddTile(t)
				tdsSeed.tileList.append(t)

			if turnTest:
				###########################################################

				tileClass = "PreTurnKeyRow"
				nextClass = "PreTurnLatchRow"

				t = TileType(True)
				t.name = ":".join(["growthAcivatePreJump", travelDir])
				t.sides[secondaryDirection] = growthActivatePreJumpGlue(travelDir)
				t.sides[primaryDirection] = growthActivatePreJumpGlue(travelDir)
				t.sides[primaryDirectionReverse] = growthActivatePropGlue(travelDir)
				t.color = "Magenta"
				tdsOut.AddTile(t)

				t = TileType(True)
				t.name = ":".join([travelDir, tileClass])
				t.sides[primaryDirection] = Glue(2, ":".join([travelDir, nextClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([travelDir, tileClass, travelDir]))
				t.sides[signalingDirection] = Glue(2, ":".join(["Turn", "Key0", travelDir]))
				t.sides[secondaryDirectionReverse] = growthActivatePreJumpGlue(travelDir)
				tdsOut.AddTile(t)

				t = TileType(True)
				t.name = ":".join(["Turn", "Key0", travelDir])
				t.sides[nonSignalingDirection] = Glue(2, ":".join(["Turn", "Key0", travelDir]))
				t.sides[primaryDirection] = Glue(2, ":".join(["Turn", "Key1", travelDir]))
				tdsOut.AddTile(t)

				t = TileType(True)
				t.name = ":".join(["Turn", "Key1", travelDir])
				t.sides[primaryDirectionReverse] = Glue(2, ":".join(["Turn", "Key1", travelDir]))
				t.sides[nonSignalingDirection] = Glue(1, ":".join(["Turn", "Key", travelDir]))
				tdsOut.AddTile(t)

				for role in validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.color = roleColor(role)
					if role == "StartPadding":
						t.sides[signalingDirection] = Glue(2, ":".join(["Turn", "Key0", travelDir]))
					tdsOut.AddTile(t)

				##############################################################

				tileClass = "PreTurnLatchRow"
				nextClass = "Unused"

				t = TileType(True)
				t.name = ":".join(["growthAcivateBandJump", travelDir])
				t.sides[secondaryDirection] = growthActivateBandJumpGlue(travelDir)
				t.sides[primaryDirection] = growthActivatePostJumpGlue(travelDir)
				t.sides[primaryDirectionReverse] = growthActivatePreJumpGlue(travelDir)
				t.sides[signalingDirection] = Glue(2, ":".join(["BandJump1", travelDir]))
				t.color = "Red"
				tdsOut.AddTile(t)

				if(travelDir in band1 or travelDir in band0xband1 or travelDir in band2xband1):
					t = TileType(True)
					t.name = ":".join(["growthActivateBandJump1", travelDir])
					t.sides[nonSignalingDirection] = Glue(2, ":".join(["BandJump1", travelDir]))
					t.sides[signalingDirection] = Glue(2, ":".join(["BandJump2", crossBandSignals[travelDir], travelDir]))
					t.color = "Red"
					tdsOut.AddTile(t)

					if(travelDir == crossBandSignals[travelDir].split("x")[0]):
						t = TileType(True)
						t.name = ":".join(["growthActivateBandJump2", crossBandSignals[travelDir], travelDir])
						t.sides[signalingDirection] = Glue(2, ":".join(["BandJump2", crossBandSignals[travelDir], crossBandSignals[travelDir].split("x")[1]]))
						t.sides[nonSignalingDirection] = Glue(2, ":".join(["BandJump2", crossBandSignals[travelDir], travelDir]))
						t.color = "Yellow"
						tdsOut.AddTile(t)

				t = TileType(True)
				t.name = ":".join([travelDir, tileClass, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([travelDir, tileClass, "Turn", travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([travelDir, tileClass, travelDir]))
				t.sides[signalingDirection] = Glue(1, ":".join(["Turn", "Key", travelDir]))
				t.sides[secondaryDirectionReverse] = growthActivateBandJumpGlue(travelDir)
				tdsOut.AddTile(t)

				for role in validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(1, ":".join([role, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.color = roleColor(role)
					if role == "StartPadding":
						t.sides[signalingDirection] = Glue(1, ":".join(["Turn", "Key", travelDir]))
						t.sides[primaryDirection] = Glue(1, ":".join([role, "GenerateKey", travelDir]))
						t.sides[secondaryDirectionReverse] = Glue(0, "")
					if role == "Group0":
						t.sides[signalingDirection] = Glue(2, ":".join(["Turn", "Key0", travelDir]))
					tdsOut.AddTile(t)

				################################################################

				tileClass = "RoT"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(1, ":".join([role, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, travelDir]))
					t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				################################################################

				tileClass = "LoT"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(1, ":".join([role, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, travelDir]))
					t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
					if(role == travelDir):
						t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				#################################################################

				tileClass = "Turn"

				t = TileType(True)
				t.name = ":".join(["growthAcivatePostJump", travelDir])
				t.sides[secondaryDirection] = growthActivatePostJumpGlue(travelDir)
				t.sides[primaryDirection] = growthActivatePropGlue(travelDir)
				t.sides[primaryDirectionReverse] = growthActivatePostJumpGlue(travelDir)
				t.color = "Magenta"
				tdsOut.AddTile(t)

				role = travelDir
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, "PreTurnLatchRow", tileClass, travelDir]))
				t.sides[secondaryDirection] = Glue(1, ":".join(["PassTurn", travelDir]))
				t.sides[primaryDirection] = Glue(1, ":".join([role, travelDir]))
				t.sides[signalingDirection] = Glue(2, ":".join([role, "Signal1", travelDir]))
				t.sides[secondaryDirectionReverse] = growthActivatePostJumpGlue(travelDir)
				t.color = "Cyan"
				tdsOut.AddTile(t)

				for role in validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.sides[secondaryDirectionReverse] = Glue(1, ":".join(["LoT", travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join(["PassTurn", travelDir]))
					t.sides[primaryDirection] = Glue(1, ":".join([role, travelDir]))
					t.sides[signalingDirection] = Glue(2, ":".join([role, "Signal1", travelDir]))
					if role == "End":
						t.sides[secondaryDirection] = Glue(2, ":".join(["PreferredDirectionKeyGen0", travelDir]))
					t.color = "Cyan"
					tdsOut.AddTile(t)

				#################################################################

				tileClass = "PassTurn"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(2, ":".join([role, "Turn", travelDir]))
					t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, "GenerateKey", travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				#################################################################

				tileClass = "Latch"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[signalingDirection] = Glue(1, ":".join(["Turn", "Key", travelDir]))
					t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join(["GenerateKey", travelDir]))
					t.sides[primaryDirection] = Glue(1, ":".join([role, "GenerateKey", travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				#################################################################

				tileClass = "GenerateKey"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[signalingDirection] = Glue(2, ":".join(["Turn", "Key0", travelDir]))
					t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, travelDir]))
					t.sides[secondaryDirectionReverse] = Glue(1, ":".join(["GenerateKey", travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join(["RoT", travelDir]))
					t.sides[primaryDirection] = Glue(1, ":".join([role, travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				##################################################################

				tileClass = "PreferredDirectionKeyGen0"
				nextClass = "PreferredDirectionKeyGen1End"

				t = TileType(True)
				t.name = ":".join([tileClass, travelDir])
				t.sides[secondaryDirectionReverse] = Glue(2, ":".join([tileClass, travelDir]))
				t.sides[primaryDirection] = Glue(2, ":".join([nextClass, travelDir]))
				t.color = "Red"
				tdsOut.AddTile(t)

				##################################################################

				tileClass = "PreferredDirectionKeyGen1End"
				keyClass = "PreferredDirectionKey1"

				t = TileType(True)
				t.name = ":".join([tileClass, travelDir])
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([tileClass, travelDir]))
				t.sides[secondaryDirectionReverse] = Glue(1, ":".join([keyClass, travelDir]))
				t.color = "Red"
				tdsOut.AddTile(t)

				##################################################################

				tileClass = "PreferredDirectionKeyGen1Start"
				nextClass = "PreferredDirectionKeyGen2"

				t = TileType(True)
				t.name = ":".join([tileClass, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([nextClass, travelDir]))
				t.sides[nonSignalingDirection] = Glue(2, ":".join([tileClass, travelDir]))
				t.color = "Red"
				tdsOut.AddTile(t)

				##################################################################

				tileClass = "PreferredDirectionKeyGen2"
				keyClass = "PreferredDirectionKey2"

				t = TileType(True)
				t.name = ":".join([tileClass, travelDir])
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([tileClass, travelDir]))
				t.sides[nonSignalingDirection] = Glue(1, ":".join([keyClass, travelDir]))
				t.color = "Red"
				tdsOut.AddTile(t)

				#################################################################

				if(travelDir in crossBandSignals.keys()):
					signalLength = crossBandOffset[crossBandSignals[travelDir]]
					#print "{}\t{}\t{}".format(crossBandSignals[travelDir], travelDir, signalLength)

					signalNum = 1
					while(signalNum < signalLength):
						tileClass = "Signal{}".format(signalNum)
						nextClass = "Signal{}".format(signalNum+1)

						for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
							t = TileType(True)
							t.name = ":".join([role, tileClass, travelDir])
							t.sides[nonSignalingDirection] = Glue(2, ":".join([role, tileClass, travelDir]))
							t.sides[signalingDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
							t.color = roleColor(role)
							tdsOut.AddTile(t)
						
						signalNum += 1

					prevClass = "Signal{}".format(signalLength)
					tileClass = "SignalTransfer"
					if travelDir in band0 or travelDir in band1xband2:
						for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
							t = TileType(True)
							t.name = ":".join([role, tileClass, crossBandSignals[travelDir]])
							t.sides[nonSignalingDirection] = Glue(2, ":".join([role, prevClass, travelDir]))
							otherSignal = [x for x in crossBandSignals[travelDir].split("x")  if x != travelDir]
							if role == travelDir:
								role = otherSignal[0]
							t.sides[signalingDirection] = Glue(2, ":".join([role, prevClass, otherSignal[0]]))
							t.color = "Green"
							tdsOut.AddTile(t)

				###################################################################

				"""tileClass = "SignalTransfer"

				try:
					t = TileType(True)
					t.name = ":".join([travelDir, "SignalTransfer", crossBandSignals[travelDir]])
					t.sides[nonSignalingDirection] = Glue(2, ":".join([travelDir, "SignalTransfer", travelDir]))
					t.sides[signalingDirection] = Glue(2, ":".join([crossBandSignals[travelDir], "Transfer", crossBandSignals[travelDir]]))
					tdsOut.AddTile(t)

					for role in validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
						t = TileType(True)
						t.name = ":".join([role, tileClass, travelDir])
						t.sides[nonSignalingDirection] = Glue(2, ":".join([role, "SignalTransfer", travelDir]))
						t.sides[signalingDirection] = Glue(2, ":".join([role, "Transfer", crossBandSignals[travelDir]]))
						t.color = roleColor(role)
						tdsOut.AddTile(t)
				except KeyError, e:
					doNothing = 1"""

				###################################################################

				tileClass = "C-Latch"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[signalingDirection] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.sides[nonSignalingDirection] = Glue(1, ":".join([role, "Latch-Active", oppositeDir]))
					t.sides[primaryDirection] = Glue(1, ":".join(["NonPrefDirKey", oppositeDir]))
					if role == travelDir:
						t.sides[nonSignalingDirection] = Glue(1, ":".join([oppositeDir, "Latch-Active", oppositeDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				###################################################################

				tileClass = "PrefDirLatchRow-2"
				nextClass = "PrefDirLatchRow-1"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, travelDir]))
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join(["PreferredDirectionKey1", travelDir]))
					t.sides[secondaryDirectionReverse] = Glue(1, ":".join(["PreferredDirectionKey1", travelDir]))
					#t.sides[nonSignalingDirection] = Glue(2, ":".join(["PrefDirKeyGen", travelDir]))
					if role == travelDir:
						t.sides[signalingDirection] = Glue(2, ":".join(["PreferredDirectionKeyGen1Start", travelDir]))
						t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				###################################################################

				tileClass = "PrefDirLatchRow-1"
				nextClass = "PrefDirLatchRow"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.sides[primaryDirection] = Glue(1, ":".join([role, nextClass, travelDir]))
					#t.sides[secondaryDirection] = Glue(1, ":".join(["PreferredDirectionKey2", travelDir]))
					#t.sides[secondaryDirectionReverse] = Glue(1, ":".join(["PreferredDirectionKey2", travelDir]))
					t.sides[nonSignalingDirection] = Glue(2, ":".join(["PrefDirKeyGen", travelDir]))
					if role == travelDir:
						t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				###################################################################

				tileClass = "PrefDirLatchRow"
				nextClass = "PrefDirLatchConnection"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, tileClass, travelDir]))
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, oppositeDir]))
					t.sides[nonSignalingDirection] = Glue(1, ":".join(["PrefDirKey", travelDir]))
					if role == travelDir:
						t.sides[primaryDirection] = Glue(2, ":".join([oppositeDir, nextClass, oppositeDir]))
						t.sides[secondaryDirectionReverse] = growthActivateSwitchHelperGlue()
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				###################################################################

				tileClass = "PrefDirKeyGen"
				nextClass = "PrefDirKey"

				t = TileType(True)
				t.name = ":".join([tileClass, travelDir])
				t.sides[signalingDirection] = Glue(2, ":".join([tileClass, travelDir]))
				t.sides[primaryDirection] = Glue(2, ":".join([nextClass, travelDir]))
				t.color = "Cyan"
				tdsOut.AddTile(t)

				###################################################################

				tileClass = "PrefDirKey"

				t = TileType(True)
				t.name = ":".join([tileClass, travelDir])
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([tileClass, travelDir]))
				t.sides[signalingDirection] = Glue(1, ":".join([tileClass, travelDir]))
				t.color = "Green"
				tdsOut.AddTile(t)


		else:
			tileClass = "PreTurn0"
			nextClass = "TurnEndFork"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				if turnTest:
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
				if instructionTest:
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				if(role == travelDir):
					t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			###################################################################

			if turnTest:

				tileClass = "TurnEndFork"
				nextClass = "PrefDirLatchConnection"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.sides[nonSignalingDirection] = Glue(2, ":".join([role, "NonPrefDirPath0", travelDir]))
					if(role == travelDir):
						t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				###################################################################

				tileClass = "NonPrefDirPath0"
				nextClass = "NonPrefDirPath1"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[signalingDirection] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.sides[nonSignalingDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				###################################################################

				tileClass = "NonPrefDirPath1"
				nextClass = "NonPrefDirPath2"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[signalingDirection] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				###################################################################

				tileClass = "NonPrefDirPath2"
				nextClass = "NonPrefDirPath3"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				###################################################################

				tileClass = "NonPrefDirPath3"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.sides[primaryDirection] = Glue(1, ":".join([role, travelDir]))
					if role == "End":
						t.sides[primaryDirection] = Glue(2, ":".join([role, "Turn", travelDir]))

					t.color = roleColor(role)
					tdsOut.AddTile(t)

				###################################################################

				tileClass = "Turn"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.sides[primaryDirection] = Glue(1, ":".join([role, travelDir]))
					if role != travelDir:
						t.sides[secondaryDirectionReverse] = Glue(1, ":".join(["PassTurn", travelDir]))
					else:
						t.sides[primaryDirection] = Glue(2, ":".join([role, "NonPrefDirLatch-3", travelDir]))
					if role != "End":
						t.sides[secondaryDirection] = Glue(1, ":".join(["LoT", travelDir]))
					t.color = "Cyan"
					tdsOut.AddTile(t)

				####################################################################

				tileClass = "PassTurn"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					if role != "End":
						t = TileType(True)
						t.name = ":".join([role, tileClass, travelDir])
						t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, travelDir]))
						t.sides[primaryDirection] = Glue(2, ":".join([role, "Turn", travelDir]))
						t.sides[secondaryDirection] = Glue(1, ":".join(["PassTurn", travelDir]))
						t.sides[secondaryDirectionReverse] = Glue(1, ":".join(["RoT", travelDir]))
						t.color = roleColor(role)
						tdsOut.AddTile(t)

				####################################################################

				tileClass = "RoT"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(1, ":".join([role, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, travelDir]))
					t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				#####################################################################

				tileClass = "LoT"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(1, ":".join([role, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, travelDir]))
					t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)


				#####################################################################

				tileClass = "NonPrefDirLatch-3"
				nextClass = "NonPrefDirLatch-2"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, travelDir]))
					t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
					if role == travelDir:
						t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
						t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				#####################################################################

				"""tileClass = "NonPrefDirLatch-3"
				nextClass = "NonPrefDirLatch-2"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					if role == travelDir:
						t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
						t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)"""

				#####################################################################

				tileClass = "NonPrefDirLatch-2"
				nextClass = "NonPrefDirLatch-1"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					if role == travelDir:
						t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
						t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				#####################################################################

				tileClass = "NonPrefDirLatch-1"
				nextClass = "NonPrefDirLatch"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					t.sides[signalingDirection] = Glue(2, ":".join(["NonPrefDirKeyGen", travelDir]))
					"""if role == travelDir:
						t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
						t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
					"""
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				######################################################################

				tileClass = "NonPrefDirLatch"
				nextClass = "Latch-Active"

				for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
					t = TileType(True)
					t.name = ":".join([role, tileClass, travelDir])
					t.sides[signalingDirection] = Glue(1, ":".join([role, nextClass, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
					if role == travelDir:
						t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
						t.sides[signalingDirection] = Glue(1, ":".join([role, nextClass, travelDir]))
					t.color = roleColor(role)
					tdsOut.AddTile(t)

				########################################################################

				t = TileType(True)
				t.name = ":".join(["NonPrefDirKey", travelDir])
				t.sides[nonSignalingDirection] = Glue(2, ":".join(["NonPrefDirKeyGen", travelDir]))
				t.sides[primaryDirection] = Glue(1, ":".join(["NonPrefDirKey", travelDir]))
				t.color = "Green"
				tdsOut.AddTile(t)

def generateInstructionTiles():
	maxGroup = sorted(groups)[-1]
	maxGroupNum = int(maxGroup.replace("Group", ""))
	maxGroupStopRole = "EndInstructionSet"

	print "TravelDir\tThisGroup\tStopRole"

	for travelDir in DirMods:
			
		dirs = PSTdir[travelDir]
		primaryDirection = DirAbbrevArray.index(dirs[0])
		secondaryDirection = DirAbbrevArray.index(dirs[1])
		tertiaryDirection = DirAbbrevArray.index(dirs[2])
		primaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[0]])
		secondaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[1]])
		tertiaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[2]])

		thisGroup = instructionGroups[travelDir]
		thisGroupNum = int(thisGroup.replace("Group", ""))

		if thisGroupNum == maxGroupNum:
			thisGroupStopRole = maxGroupStopRole
		else:
			thisGroupStopRole = "Group" + str(thisGroupNum + 1)

		print travelDir + "\t\t" + thisGroup + "\t\t" + thisGroupStopRole

		instrID = "Instr"

		if instructionTest:

			tileClass = "PostInstructions"
			nextClass = "PreTurn0"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				t.sides[tertiaryDirection] = Glue(2, ":".join([role, "InactiveRise", travelDir]))
				if(role == travelDir):
					t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "PostInstLatchInterface"
			nextClass = "PostInstructions"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				if(role == travelDir):
					t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "InactiveRise"
			nextClass = "InactiveTurn"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[tertiaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				t.sides[tertiaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "InactiveTurn"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[tertiaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, instrID, travelDir]))
				if role == thisGroup:
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, "DoRole(Inactive)", travelDir]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "DoRole(Inactive)"
			nextClass = "ReceiveRole"
			doneClass = "InactiveDone-4"
			helperClass = ":".join(["LoTRoTHelper", "Inactive"])

			for role in validRoles + singletonTiles + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([role, tileClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, instrID, travelDir]))
				t.sides[secondaryDirection] = Glue(1, ":".join([nextClass, travelDir]))
				t.sides[secondaryDirectionReverse] = Glue(1, ":".join(["LoT", travelDir]))
				t.sides[tertiaryDirection] = Glue(2, ":".join([helperClass, travelDir]))
				if role == thisGroupStopRole:
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, doneClass, travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join(["RoT", travelDir]))
				t.color = "Cyan"
				tdsOut.AddTile(t)

			tileClass = "ReceiveRole"
			nextClass = "DoRole(Inactive)"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirection] = Glue(1, ":".join([role, instrID, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, nextClass, travelDir]))
				t.sides[secondaryDirection] = Glue(1, ":".join(["RoT", travelDir]))
				t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			###############################################
			######		Copied from Turn 		###########
			###############################################

			tileClass = "RoT"
			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, instrID, travelDir])
				t.sides[primaryDirection] = Glue(1, ":".join([role, instrID, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, instrID, travelDir]))
				t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
				t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
				t.sides[tertiaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			if travelDir == "NU":
				for tileClass in ["{}Query".format(d) for d in DirAbbrevArray]:
					for role in validRoles + singletonTiles + baseTileNames + list(groups):
						t = TileType(True)
						t.name = ":".join([role, tileClass, instrID, travelDir])
						t.sides[primaryDirection] = Glue(1, ":".join([role, instrID, travelDir]))
						t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, instrID, travelDir]))
						t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
						t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
						t.sides[tertiaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
						if role == "Interface1" and "Query" in tileClass:
							t.sides[tertiaryDirectionReverse] = Glue(2, "QueryStart")
						if role in baseTileNames + ["Prop"]:
							t.sides[tertiaryDirectionReverse] = Glue(1, role)
							if "Query" in tileClass:
								if role == "TD":
									t.sides[tertiaryDirectionReverse] = Glue(1, "{}:{}".format(role, tileClass))
						if role == "Interface2":
							t.sides[secondaryDirection] = Glue(1, ":".join(["RoT", travelDir]))
						t.color = "Yellow"
						tdsOut.AddTile(t)

				for tileClass in ["Initialize"]:
					for role in validRoles + singletonTiles + baseTileNames + list(groups):
						t = TileType(True)
						t.name = ":".join([role, tileClass, instrID, travelDir])
						t.sides[primaryDirection] = Glue(1, ":".join([role, instrID, travelDir]))
						t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, instrID, travelDir]))
						t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
						t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
						t.sides[tertiaryDirectionReverse] = Glue(1, "CallbackHelper")
						if role == "Interface2":
							t.sides[tertiaryDirectionReverse] = Glue(2, "InitializationKeyGen")
							t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, "Active", travelDir]))
						if role == "Interface3":
							t.sides[secondaryDirection] = Glue(1, ":".join(["RoT", travelDir]))
							t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, "Active", travelDir]))
						tdsOut.AddTile(t)
					for role in initTileNamesActive + ["Prop"]:
						t = TileType(True)
						t.name = ":".join([role, tileClass,"Active", instrID, travelDir])
						t.sides[primaryDirection] = Glue(1, ":".join([role, instrID, travelDir]))
						t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, instrID, travelDir]))
						t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, "Active", travelDir]))
						t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, "Active", travelDir]))
						t.sides[tertiaryDirectionReverse] = Glue(1, ":".join([role, "Active"]))
						if role == "Prop":
							t.sides[tertiaryDirectionReverse] = Glue()
						tdsOut.AddTile(t)

				for role in validRoles + [travelDir] + list(groups) + ["StartPadding"]:
					t = TileType(True)
					t.name = ":".join([role, "Callback", travelDir])
					t.sides[primaryDirection] = Glue(1, ":".join([role, instrID, travelDir]))
					t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, instrID, travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join(["Callback", travelDir]))
					t.sides[secondaryDirectionReverse] = Glue(1, ":".join(["Callback", travelDir]))
					t.sides[tertiaryDirectionReverse] = Glue(1, "CallbackHelper")
					if(role == travelDir):
						t.sides[secondaryDirectionReverse] = growthActivateStartGlue(travelDir)
						t.sides[tertiaryDirectionReverse] = Glue(1, "CallbackEnd")
					t.color = "Yellow"
					tdsOut.AddTile(t)



			################################################################

			tileClass = "LoT"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, instrID, travelDir])
				t.sides[primaryDirection] = Glue(1, ":".join([role, instrID, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, instrID, travelDir]))
				t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
				t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
				t.sides[tertiaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
				if(role == travelDir):
					t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			############################################################################################

			tileClass = "InactiveDone-4"
			nextClass = "InactiveDone-3"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, instrID, travelDir])
				t.sides[primaryDirection] = Glue(1, ":".join([role, instrID, travelDir]))
				t.sides[secondaryDirection] = Glue(1, ":".join([tileClass, travelDir]))
				t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, nextClass, travelDir]))
				if role == thisGroupStopRole:
					t.sides[primaryDirection] = Glue(2, ":".join([role, tileClass, travelDir]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			############################################################################################

			tileClass = "Key"

			t = TileType(True)
			t.name = ":".join([role, tileClass, instrID, travelDir])
			t.sides[tertiaryDirection] = Glue(2, ":".join(["KeyGen", instrID, travelDir]))
			t.sides[primaryDirectionReverse] = Glue(1, ":".join([tileClass, instrID, travelDir]))
			t.color = "Green"
			tdsOut.AddTile(t)


			tileClass = "InactiveDone-3"
			nextClass = "InactiveDone-2"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, instrID, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([role, tileClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, nextClass, travelDir]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "InactiveDone-2"
			nextClass = "InactiveDone-1"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, instrID, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([role, tileClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, nextClass, travelDir]))
				t.sides[tertiaryDirectionReverse] = Glue(2, ":".join(["KeyGen", instrID, travelDir]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "InactiveDone-1"
			nextClass = "InactiveDone"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, instrID, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([role, tileClass, travelDir]))
				t.sides[tertiaryDirectionReverse] = Glue(1, ":".join([role, nextClass, travelDir]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			nextClass = "InactiveDone"
			tileClass = "InactiveLatch"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, instrID, travelDir])
				t.sides[primaryDirection] = Glue(1, ":".join(["Key", instrID, travelDir]))
				t.sides[tertiaryDirection] = Glue(1, ":".join([role, nextClass, travelDir]))
				t.sides[tertiaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "PreInstructions"
			prevClass = "Intersection+3"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, instrID, travelDir])
				t.sides[primaryDirection] = Glue(1, ":".join([role, instrID, travelDir]))
				t.sides[tertiaryDirection] = Glue(2, ":".join([role, "InactiveLatch", travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				if(role == travelDir):
					t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
				if role == thisGroup:
					if travelDir != "NU":
						t.sides[primaryDirection] = Glue(2, ":".join([role, "DoRole(Active)", travelDir]))
					else:
						TODO = 1
						t.sides[primaryDirection] = Glue(2, ":".join([role, "DoRole(Active)", travelDir]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "DoRole(Active)"
			nextClass = "ReceiveRole(Active)"
			doneClass = "ActiveLatch-1"
			#helperClass = ":".join(["LoTRoTHelper", "Active"])
			helperClass = "ActiveKeyGenHelper"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				t.sides[primaryDirection] = Glue(1, ":".join([role, instrID, travelDir]))
				t.sides[secondaryDirection] = Glue(1, ":".join([nextClass, travelDir]))
				t.sides[secondaryDirectionReverse] = Glue(1, ":".join(["LoT", travelDir]))
				#t.sides[tertiaryDirection] = Glue(2, ":".join([helperClass, travelDir]))
				if role == thisGroupStopRole:
					t.sides[primaryDirection] = Glue(2, ":".join([role, doneClass, travelDir]))
					t.sides[secondaryDirection] = Glue(1, ":".join(["RoT", travelDir]))
					t.sides[tertiaryDirection] = Glue(2, ":".join([helperClass, travelDir]))
				if role == "Intersection" and travelDir == "NU":
					t.sides[secondaryDirectionReverse] = Glue(1, ":".join(["Callback", travelDir]))
					t.sides[tertiaryDirectionReverse] = Glue(1, "CallbackHelper")
				t.color = "Cyan"
				tdsOut.AddTile(t)

			tileClass = "ReceiveRole(Active)"
			nextClass = "DoRole(Active)"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, instrID, travelDir]))
				t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
				if travelDir == "NU" and ("Query" in role or role == "Initialize"):
					t.sides[secondaryDirection] = Glue(1, ":".join([role, travelDir]))
					t.sides[tertiaryDirectionReverse] = Glue(2, "Test{}".format(tertiaryDirection))
				if(travelDir == "NU" and role == "Initialize"):
					t.sides[tertiaryDirectionReverse] = Glue(1, "CallbackHelper")
				else:
					t.sides[secondaryDirection] = Glue(1, ":".join(["RoT", travelDir]))
				t.sides[secondaryDirectionReverse] = Glue(1, ":".join([tileClass, travelDir]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "ActiveKeyGenHelper"
			keyGenClass = "Active_KeyGen"
			keyClass1 = "Active_Key1"
			keyClass2 = "Active_Key2"

			t = TileType(True)
			t.name = ":".join([tileClass, travelDir])
			t.sides[tertiaryDirectionReverse] = Glue(2, ":".join([tileClass, travelDir]))
			t.sides[secondaryDirection] = Glue(1, ":".join([keyGenClass, travelDir]))
			t.sides[secondaryDirectionReverse] = Glue(1, ":".join([keyGenClass, travelDir]))
			t.sides[primaryDirection] = Glue(2, ":".join([keyClass1, travelDir]))
			t.color = "Green"
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = ":".join([keyGenClass, "RoT", travelDir])
			t.sides[tertiaryDirectionReverse] = Glue(1, ":".join(["RoT", travelDir]))
			t.sides[secondaryDirection] = Glue(1, ":".join([keyGenClass, travelDir]))
			t.sides[secondaryDirectionReverse] = Glue(1, ":".join([keyGenClass, travelDir]))
			t.sides[primaryDirection] = Glue(2, ":".join([keyClass1, travelDir]))
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = ":".join([keyGenClass, "LoT", travelDir])
			t.sides[tertiaryDirectionReverse] = Glue(1, ":".join(["LoT", travelDir]))
			t.sides[secondaryDirection] = Glue(1, ":".join([keyGenClass, travelDir]))
			t.sides[secondaryDirectionReverse] = Glue(1, ":".join([keyGenClass, travelDir]))
			t.sides[primaryDirection] = Glue(2, ":".join([keyClass1, travelDir]))
			if(role == travelDir):
				t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = ":".join([keyClass1, travelDir])
			t.sides[primaryDirectionReverse] = Glue(2, ":".join([keyClass1, travelDir]))
			t.sides[primaryDirection] = Glue(2, ":".join([keyClass2, travelDir]))
			t.sides[tertiaryDirectionReverse] = Glue(1, ":".join([keyClass1, travelDir]))
			t.color = "Cyan"
			tdsOut.AddTile(t)

			t = TileType(True)
			t.name = ":".join([keyClass2, travelDir])
			t.sides[primaryDirectionReverse] = Glue(2, ":".join([keyClass2, travelDir]))
			t.sides[tertiaryDirectionReverse] = Glue(1, ":".join([keyClass2, travelDir]))
			t.color = "Magenta"
			tdsOut.AddTile(t)

			tileClass = "ActiveLatch-1"
			nextClass = "ActiveLatch"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):

				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirection] = Glue(1, ":".join([role, nextClass, travelDir]))
				t.sides[tertiaryDirection] = Glue(1, ":".join([keyClass1, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, instrID, travelDir]))
				if role == thisGroupStopRole:
					t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				if(role == travelDir):
					t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "KeyGenActive"
			nextClass = "KeyActive"

			t = TileType(True)
			t.name = ":".join([tileClass, travelDir])
			t.sides[tertiaryDirectionReverse] = Glue(2, ":".join([tileClass, instrID, travelDir]))
			t.sides[primaryDirection] = Glue(2, ":".join([nextClass, instrID, travelDir]))
			t.color = "Cyan"
			tdsOut.AddTile(t)

			tileClass = "KeyActive"

			t = TileType(True)
			t.name = ":".join([tileClass, travelDir])
			t.sides[tertiaryDirectionReverse] = Glue(1, ":".join([tileClass, instrID, travelDir]))
			t.sides[primaryDirectionReverse] = Glue(2, ":".join([tileClass, instrID, travelDir]))
			t.color = "Green"
			tdsOut.AddTile(t)

			tileClass = "ActiveLatch"
			nextClass = "PostInstLatchInterface"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, tileClass, travelDir]))
				t.sides[tertiaryDirection] = Glue(1, ":".join([keyClass2, travelDir]))
				if(role == travelDir):
					t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "Intersection+3"
			nextClass = "PreInstructions"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				if(role == travelDir):
					t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "Intersection+2"
			nextClass = "Intersection+3"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				if(role == travelDir):
					t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			tileClass = "Intersection+1"
			nextClass = "Intersection+2"

			for role in [travelDir] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups):
				t = TileType(True)
				t.name = ":".join([role, tileClass, travelDir])
				t.sides[primaryDirection] = Glue(2, ":".join([role, nextClass, travelDir]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, tileClass, travelDir]))
				if(role == travelDir):
					t.sides[secondaryDirectionReverse] = growthActivateHelperGlue(travelDir)
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			t = TileType(True)
			t.name = ":".join(["GrowthActivationProp", travelDir])
			t.sides[primaryDirection] = growthActivatePropGlue(travelDir)
			t.sides[primaryDirectionReverse] = growthActivatePropGlue(travelDir)
			t.sides[secondaryDirection] = growthActivateHelperGlue(travelDir)
			t.color = "Cyan"
			tdsOut.AddTile(t)

			if(travelDir[0] in majorDirs):
				t = TileType(True)
				t.name = ":".join(["GrowthActivationPropSwitch", travelDir, oppositeByDirMod[travelDir]])
				t.sides[primaryDirection] = growthActivatePropGlue(oppositeByDirMod[travelDir])
				t.sides[primaryDirectionReverse] = growthActivatePropGlue(travelDir)
				t.sides[secondaryDirection] = growthActivateSwitchHelperGlue()
				t.color = "Blue"
				tdsOut.AddTile(t)

			if(travelDir == "NU"):
				t = TileType(True)
				t.name = "GrowthActivationStart"
				t.sides[tertiaryDirectionReverse] = Glue(1, "CallbackTurn")
				t.sides[secondaryDirection] = growthActivateStartGlue(travelDir)
				t.sides[primaryDirection] = growthActivatePropGlue(travelDir)
				t.sides[primaryDirectionReverse] = growthActivatePropGlue(travelDir)
				t.color = "Yellow"
				tdsOut.AddTile(t)

def generateIntersectionTiles():
	#print Intersections

	d1External = ""
	d2External = ""

	for xDescriptor in set(Intersections.values()):
		data = xDescriptor.split("x")



		dirMod1 = data[0]
		dirMod1_out = oppositeByDirMod[dirMod1]
		face1 = dirMod1[1]
		
		dirMod2 = data[1]
		dirMod2_out = oppositeByDirMod[dirMod2]
		face2 = dirMod2[1]
		
		d1Internal = DirAbbrevArray.index(data[0][0])
		d2Internal = DirAbbrevArray.index(data[1][0])
		d1External = DirAbbrevArray.index(oppositeByDir[data[0][0]])
		d2External = DirAbbrevArray.index(oppositeByDir[data[1][0]])
		dirsTemp = PSTdir[dirMod1]
		secondaryDirection = DirAbbrevArray.index(dirsTemp[1])
		secondaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirsTemp[1]]) 
		#print("{}: Face={}\tTarget={}\tExternal={}".format(xDescriptor, face1, ExternalCommunicationOut[face1], dirMod2_out))
		#print("{}: Face={}\tTarget={}\tExternal={}\n".format(xDescriptor, face2, ExternalCommunicationOut[face2], dirMod1_out))

		#print "\t".join([xDescriptor, dirMod1, str(d1Internal), str(d1External), dirMod2, str(d2Internal), str(d2External)])
		#print "\t".join([xDescriptor, dirMod1, dirMod1_out, dirMod2, dirMod2_out])
		tileClass = "Intersection"
		nextClassInternal = "Intersection+1"
		nextClassExternalOut = "External_Out+1"
		nextClassExternalIn = "External_In-1"

		for role in [xDescriptor] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups): 
			t = TileType(True)
			t.name = ":".join([role, tileClass, xDescriptor])
			t.sides[d1Internal] = Glue(2, ":".join([role, nextClassInternal, dirMod1]))
			t.sides[d2Internal] = Glue(2, ":".join([role, nextClassInternal, dirMod2]))

			if(dirMod1_out in ExternalCommunicationOut.values()):
				if(role == xDescriptor):
					print("{}: Output => {}".format(xDescriptor, dirMod1_out))
				t.sides[d1External] = Glue(1, ":".join([role, nextClassExternalOut, dirMod1_out]))
			if(dirMod1_out in ExternalCommunicationIn.values()):
				if(role == xDescriptor):
					print("{}: Input <= {}".format(xDescriptor, dirMod1_out))
					t.sides[d1External] = Glue(2, ":".join([Intersections[dirMod1_out], nextClassExternalIn, dirMod1]))
				else:
					t.sides[d1External] = Glue(2, ":".join([role, nextClassExternalIn, dirMod1]))

			if(dirMod2_out in ExternalCommunicationOut.values()):
				if(role == xDescriptor):
					print("{}: Output => {}".format(xDescriptor, dirMod2_out))
				t.sides[d2External] = Glue(1, ":".join([role, nextClassExternalOut, dirMod2_out]))
			if(dirMod2_out in ExternalCommunicationIn.values()):
				if(role == xDescriptor):
					print("{}: Input <= {}".format(xDescriptor, dirMod2_out))
					t.sides[d2External] = Glue(2, ":".join([Intersections[dirMod2_out], nextClassExternalIn, dirMod2]))
				else:
					t.sides[d2External] = Glue(2, ":".join([role, nextClassExternalIn, dirMod2]))

			if role == xDescriptor:
				t.sides[d1Internal] = Glue(2, ":".join([dirMod1, nextClassInternal, dirMod1]))
				t.sides[d2Internal] = Glue(2, ":".join([dirMod2, nextClassInternal, dirMod2]))
				t.sides[secondaryDirectionReverse] = Glue(1, ":".join(["ActivateExternalGrowthKeyGen", xDescriptor]))
			t.color = roleColor(role)
			tdsOut.AddTile(t)

		t = TileType(True)
		t.name = ":".join(["ActivateExternalGrowthKeyGen", xDescriptor])
		t.sides[secondaryDirection] = Glue(1, ":".join(["ActivateExternalGrowthKeyGen", xDescriptor]))
		t.sides[d1External] = Glue(2, ":".join(["ActivateExternalGrowthKeyGen", dirMod1_out]))
		t.sides[d2External] = Glue(2, ":".join(["ActivateExternalGrowthKeyGen", dirMod2_out]))
		t.sides[d1Internal] = growthActivatePropGlue(dirMod1)
		t.sides[d2Internal] = growthActivatePropGlue(dirMod2)
		t.color = "Green"
		tdsOut.AddTile(t)

# the genome will not propagate to any direction specified in inhibit
def generateExternalCommunicationTiles(inhibit=[]):
	for face, outDirMod in ExternalCommunicationOut.items():
		dirs = PSTdir[outDirMod]
		primaryDirection = DirAbbrevArray.index(dirs[0])
		secondaryDirection = DirAbbrevArray.index(dirs[1])
		tertiaryDirection = DirAbbrevArray.index(dirs[2])
		primaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[0]])
		secondaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[1]])
		tertiaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[2]])

		inDirMod = oppositeByDirMod[outDirMod]
		outIntersection = Intersections[oppositeByDirMod[outDirMod]]
		inIntersection = Intersections[inDirMod]

		for role in [outIntersection] + validRoles + singletonTiles + baseTileNames + initTileNames + list(groups): 
			inputClass = "External_Out+1"
			outputClass = "External_Out+2"

			t = TileType(True)
			t.name = ":".join([role, inputClass, outDirMod])
			if(role != "End"):
				t.sides[secondaryDirection] = Glue(1, "ActivateExternalGrowthKey")
			t.sides[secondaryDirectionReverse] = Glue(1, "ActivateExternalGrowthKey")
			t.sides[primaryDirection] = Glue(2, ":".join([role, outputClass, outDirMod]))
			t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, inputClass, outDirMod]))
			t.color = roleColor(role)
			tdsOut.AddTile(t)

			outputLength = 10

			for i in range(outputLength):
				inputClass = "External_Out+{}".format(i+2)
				outputClass = "External_Out+{}".format(i+3)

				t = TileType(True)
				t.name = ":".join([role, inputClass, outDirMod])
				t.sides[primaryDirection] = Glue(2, ":".join([role, outputClass, outDirMod]))
				t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, inputClass, outDirMod]))
				t.color = roleColor(role)
				tdsOut.AddTile(t)

			inputClass = "External_Out+{}".format(outputLength + 2)
			outputClass = "Transition"

			t = TileType(True)
			t.name = ":".join([role, inputClass, outDirMod])
			t.sides[primaryDirection] = Glue(2, ":".join([role, outputClass, outDirMod]))
			t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, inputClass, outDirMod]))
			t.color = roleColor(role)
			tdsOut.AddTile(t)

			inputClass = "Transition"
			outputClass = "External_In-3"

			t = TileType(True)
			t.name = ":".join([role, inputClass, outDirMod])
			t.sides[tertiaryDirection] = Glue(2, ":".join(["ExternalInputKeyGen", outDirMod]))
			t.sides[primaryDirection] = Glue(1, ":".join([role, outputClass, outDirMod]))
			t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, inputClass, outDirMod]))
			t.color = roleColor(role)
			tdsOut.AddTile(t)

			inputClass = "External_In-3"
			outputClass = "External_In-2"

			t = TileType(True)
			t.name = ":".join([role, inputClass, outDirMod])
			t.sides[tertiaryDirection] = Glue(1, "ExternalInputKey")
			t.sides[primaryDirection] = Glue(2, ":".join([role, outputClass, outDirMod]))
			t.sides[primaryDirectionReverse] = Glue(1, ":".join([role, inputClass, outDirMod]))
			t.color = roleColor(role)
			tdsOut.AddTile(t)

			inputClass = "External_In-2"
			outputClass = "External_In-1"

			t = TileType(True)
			t.name = ":".join([role, inputClass, outDirMod])
			if(role == outIntersection):
				t.sides[primaryDirection] = Glue(2, ":".join([inIntersection, outputClass, outDirMod]))
			else:
				t.sides[primaryDirection] = Glue(2, ":".join([role, outputClass, outDirMod]))
			t.sides[primaryDirectionReverse] = Glue(2, ":".join([role, inputClass, outDirMod]))
			t.color = roleColor(role)
			tdsOut.AddTile(t)

		t = TileType(True)
		t.name = ":".join(["ExternalInputKeyGen", outDirMod])
		t.sides[tertiaryDirectionReverse] = Glue(2, ":".join(["ExternalInputKeyGen", outDirMod]))
		t.sides[primaryDirection] = Glue(2, ":".join(["ExternalInputKeyGen", outDirMod]))
		t.color = "Green"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = ":".join(["ExternalInputKey", outDirMod])
		t.sides[primaryDirectionReverse] = Glue(2, ":".join(["ExternalInputKeyGen", outDirMod]))
		t.sides[tertiaryDirectionReverse] = Glue(1, "ExternalInputKey")
		t.color = "Yellow"
		tdsOut.AddTile(t)

		if primaryDirection not in inhibit:
			print "Non-inhibited genome activate direction: " + str(primaryDirection)
			t = TileType(True)
			t.name = ":".join(["ActivateExternalGrowthKey", outDirMod])
			t.sides[primaryDirectionReverse] = Glue(2, ":".join(["ActivateExternalGrowthKeyGen", outDirMod]))
			t.sides[secondaryDirection] = Glue(1, "ActivateExternalGrowthKey")
			t.color = "Yellow"
			tdsOut.AddTile(t)



def generateQueryTiles():
	primaryDirection = N
	primaryDirectionReverse = S
	secondaryDirection = E
	secondaryDirectionReverse = W
	tertiaryDirection = U
	tertiaryDirectionReverse = D

	bits = ["0", "1"]
	roles = ["lm", "m", "l", ""]
	validData = ["0", "1", "0l", "1l", "0lm", "1lm", "1m"]

	t = TileType(True)
	t.name = "QueryStart"
	t.sides[tertiaryDirection] = Glue(2, "QueryStart")
	t.sides[secondaryDirection] = Glue(1, "QueryStart")
	t.color = "Cyan"
	tdsOut.AddTile(t)

	####################	BUILDERS	#######################

	t = TileType(True)
	t.name = "QueryTileZERO"
	t.sides[secondaryDirectionReverse] = Glue(1, "QueryStart")
	t.sides[primaryDirection] = Glue(1, "Check:Prop")
	t.sides[secondaryDirection] = Glue(1, "0lm*")
	t.sides[tertiaryDirectionReverse] = Glue(2, "BuilderTop")
	t.sides[tertiaryDirection] = Glue(1, "Prop")
	t.color = "White"
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "BuilderTop"
	t.sides[tertiaryDirection] = Glue(2, "BuilderTop")
	t.sides[secondaryDirection] = Glue(1, "BuilderPenultimate")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "BuilderPenultimate"
	t.sides[secondaryDirectionReverse] = Glue(1, "BuilderPenultimate")
	t.sides[secondaryDirection] = Glue(1, "BuilderHelper")
	t.sides[tertiaryDirection] = Glue(1, "Build")
	t.sides[tertiaryDirectionReverse] = Glue(2, "BuilderTop")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "BuilderZero"
	t.sides[tertiaryDirection] = Glue(1, "Prop")
	t.sides[tertiaryDirectionReverse] = Glue(1, "Build")
	t.sides[secondaryDirectionReverse] = Glue(1, "0lm*")
	t.sides[secondaryDirection] = Glue(1, "0lm*")
	t.sides[primaryDirection] = Glue(1, "Check:Prop")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "BuilderHelper"
	t.sides[secondaryDirection] = Glue(1, "BuilderHelper")
	t.sides[secondaryDirectionReverse] = Glue(1, "BuilderHelper")
	t.sides[tertiaryDirectionReverse] = Glue(1, "Build")
	t.sides[tertiaryDirection] = Glue(1, "Build")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "CheckZERO"
	t.sides[secondaryDirectionReverse] = Glue(1, "0lm*")
	t.sides[secondaryDirection] = Glue(1, "0lm")
	t.sides[primaryDirection] = Glue(1, "Check:0lm")
	t.sides[tertiaryDirection] = Glue(1, "Check")
	t.sides[tertiaryDirectionReverse] = Glue(1, "Check")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "BuilderHelper->CheckPadding"
	t.sides[secondaryDirection] = Glue(1, "Padding")
	t.sides[secondaryDirectionReverse] = Glue(1, "BuilderHelper")
	t.sides[primaryDirection] = Glue(1, "Check:0")
	t.sides[tertiaryDirectionReverse] = Glue(1, "Check")
	t.sides[tertiaryDirection] = Glue(1, "Check")
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "BuilderPenultimate->CheckPadding"
	t.sides[secondaryDirection] = Glue(1, "Padding")
	t.sides[secondaryDirectionReverse] = Glue(1, "BuilderPenultimate")
	t.sides[primaryDirection] = Glue(1, "Check:0")
	t.sides[tertiaryDirectionReverse] = Glue(1, "Check")
	t.sides[tertiaryDirection] = Glue(1, "Check")
	tdsOut.AddTile(t)

	######################	PROP 		#########################

	for role in baseTileNames:
		if role == "TD":
			continue
		elif role == "Check":
			for data in validData + ["Padding"]:
				t = TileType(True)
				t.name = "Check:{}".format(data)
				t.sides[secondaryDirectionReverse] = Glue(1, data)
				t.sides[secondaryDirection] = Glue(1, data)
				if data != "Padding":
					t.sides[primaryDirection] = Glue(1, "Check:{}".format(data))
				else:
					t.sides[primaryDirection] = Glue(1, "Check:0")
				t.sides[tertiaryDirection] = Glue(1, "Check")
				t.sides[tertiaryDirectionReverse] = Glue(1, "Check")
				if "1" in data:
					t.color = "Blue"
				tdsOut.AddTile(t)
		else:
			for data in [x for x in validData if "l" in x]:
				t.name = "{}:{}:Prop".format(role, data)
				t.sides[secondaryDirectionReverse] = Glue(1, data)
				t.sides[secondaryDirection] = Glue(1, data)
				t.sides[primaryDirection] = Glue(1, "Check:Prop")
				t.sides[tertiaryDirection] = Glue(1, role)
				t.sides[tertiaryDirectionReverse] = Glue(1, "Prop")
				tdsOut.AddTile(t)

	for data in [x for x in validData if "l" not in x] + ["Padding"]:
		t.name = "{}:Prop".format(data)
		t.sides[secondaryDirectionReverse] = Glue(1, data)
		t.sides[secondaryDirection] = Glue(1, data)
		t.sides[tertiaryDirection] = Glue(1, "Prop")
		t.sides[tertiaryDirectionReverse] = Glue(1, "Prop")
		if "1" in data:
			t.color = "Blue"
		if data == "Padding":
			print t.name
			t.color = "Yellow"
		tdsOut.AddTile(t)




	#####################	INCREMENT 	#########################

	for data in [x for x in validData if "l" in x]:
		t = TileType(True)
		t.name = "Increment:{}".format(data)
		t.sides[primaryDirection] = Glue(1, "Increment")
		t.sides[secondaryDirectionReverse] = Glue(1, data)
		if "0" in data:
			t.sides[secondaryDirection] = Glue(1, data.replace("0","1"))
			t.sides[tertiaryDirectionReverse] = Glue(1, "NoCarry")
		elif "1" in data:
			t.sides[secondaryDirection] = Glue(1, "0l")
			t.sides[tertiaryDirectionReverse] = Glue(1, "Carry")
		tdsOut.AddTile(t)

	for data in [x for x in validData if "l" not in x] + ["Padding"]:
		role = "NoCarry"

		t = TileType(True)
		t.name = "Increment:{}:{}".format(data, role)
		t.sides[secondaryDirectionReverse] = Glue(1, data)
		t.sides[secondaryDirection] = Glue(1, data)
		t.sides[tertiaryDirectionReverse] = Glue(1, role)
		t.sides[tertiaryDirection] = Glue(1, role)
		tdsOut.AddTile(t)

		role = "Carry"
		
		t.name = "Increment:{}:{}".format(data, role)
		t.sides[secondaryDirectionReverse] = Glue(1, data)
		t.sides[tertiaryDirection] = Glue(1, role)
		if "1" in data:
			t.sides[secondaryDirection] = Glue(1, "0")
			t.sides[tertiaryDirectionReverse] = Glue(1, "Carry")
		elif "0" in data:
			t.sides[secondaryDirection] = Glue(1, "1")
			t.sides[tertiaryDirectionReverse] = Glue(1, "NoCarry")
		else: #data=="Padding"
			t.sides[secondaryDirection] = Glue(1, "1m")
			t.sides[tertiaryDirectionReverse] = Glue(1, "NoCarry")
		tdsOut.AddTile(t)

def generateQueryCheckTiles():
	primaryDirection = N
	primaryDirectionReverse = S
	secondaryDirection = E
	secondaryDirectionReverse = W
	tertiaryDirection = U
	tertiaryDirectionReverse = D

	bits = ["0", "1"]
	roles = ["lm", "m", "l", ""]
	validData = ["0", "1", "0l", "1l", "0lm", "1lm", "1m"]

	for data in validData:
		t = TileType(True)
		t.name = "QueryCheckSeed:{}".format(data)
		t.sides[secondaryDirection] = Glue(1, data)
		if "1" in data:
			t.color = "Blue"
		tdsOut.AddTile(t)

	for data in validData:
		t = TileType(True)
		t.name = "QueryCheck:{}:{}".format(data, "Prop")
		if "l" in data:
			t.sides[primaryDirectionReverse] = Glue(1, "Check:Prop")
		else:
			t.sides[tertiaryDirection] = Glue(1, "Check:Prop")
		t.sides[secondaryDirectionReverse] = Glue(1, data)
		t.sides[secondaryDirection] = Glue(1, data)
		if "m" not in data:
			t.sides[tertiaryDirectionReverse] = Glue(1, "Check:Prop")
		if "1" in data:
			t.color = "Blue"
		tdsOut.AddTile(t)

		for role in ["Equal", "NotEqual"]:
			t = TileType(True)
			t.name = "{}:{}:Comm".format(data, role)
			t.sides[secondaryDirection] = Glue(1, data)
			t.sides[secondaryDirectionReverse] = Glue(1, data)
			if "m" in data:
				t.sides[primaryDirection] = Glue(1, role)
				t.sides[tertiaryDirection] = Glue(1, role)
			elif "l" in data:
				t.sides[tertiaryDirectionReverse] = Glue(1, role)
				if role == "NotEqual":
					t.sides[primaryDirectionReverse] = Glue(1, "Increment")
				else:
					t.sides[primaryDirectionReverse] = Glue(1, "Query")
			else:
				t.sides[tertiaryDirection] = Glue(1, role)
				t.sides[tertiaryDirectionReverse] = Glue(1, role)
			t.color = roleColor(role)
			tdsOut.AddTile(t)



	for data in [x for x in validData if "l" in x]:
		t = TileType(True)
		t.sides[secondaryDirectionReverse] = Glue(1, data)
		t.sides[secondaryDirection] = Glue(1, data)
		for data2 in [x for x in validData if "l" in x]:
			t.name = "CheckStart:{}=?={}".format(data, data2)
			t.sides[primaryDirectionReverse] = Glue(1, "Check:{}".format(data2))
			if data == data2:
				
				if "m" in data:
					t.sides[secondaryDirection] = Glue(2, "EqualDone:lm")
				else:
					t.sides[primaryDirection] = Glue(2, "Equal")
				t.color = "Green"
			else:
				if "m" in data:
					t.sides[secondaryDirection] = Glue(2, "NotEqualDone:lm")
				else:
					t.sides[primaryDirection] = Glue(2, "NotEqual")
				t.color = "Red"
			tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "NotEqual:{}:Prop".format(data)
		t.sides[secondaryDirectionReverse] = Glue(1, data)
		t.sides[secondaryDirection] = Glue(1, data)
		t.sides[primaryDirectionReverse] = Glue(1, "Increment")
		t.sides[tertiaryDirectionReverse] = Glue(1, "NotEqual:Prop")
		tdsOut.AddTile(t)

	for data in [x for x in validData if "l" not in x and "m" not in x]:
		t = TileType(True)
		t.sides[secondaryDirectionReverse] = Glue(1, data)
		t.sides[secondaryDirection] = Glue(1, data)
		for data2 in [x for x in validData if "l" not in x and "m" not in x]:
			t.name = "Check:{}=?={}".format(data, data2)
			t.sides[primaryDirectionReverse] = Glue(1, "Check:{}".format(data2))
			if data == data2:
				t.sides[primaryDirection] = Glue(1, "Equal")
				t.color = "Green"
			else:
				t.sides[primaryDirection] = Glue(1, "NotEqual")
				t.color = "Red"
			tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "NotEqual:{}:Prop".format(data)
		t.sides[secondaryDirectionReverse] = Glue(1, data)
		t.sides[secondaryDirection] = Glue(1, data)
		t.sides[tertiaryDirection] = Glue(1, "NotEqual:Prop")
		t.sides[tertiaryDirectionReverse] = Glue(1, "NotEqual:Prop")
		tdsOut.AddTile(t)

	for data in [x for x in validData if "m" in x and "l" not in x]:
		t = TileType(True)
		t.sides[secondaryDirectionReverse] = Glue(1, data)
		t.sides[secondaryDirection] = Glue(1, data)
		for data2 in [x for x in validData if "l" not in x]:
			t.name = "Check:{}=?={}".format(data, data2)
			t.sides[primaryDirectionReverse] = Glue(1, "Check:{}".format(data2))
			if data == data2:
				t.sides[primaryDirection] = Glue(1, "EqualDone")
				t.color = "Green"
			else:
				t.sides[primaryDirection] = Glue(1, "NotEqualDone")
				t.color = "Red"
			tdsOut.AddTile(t)



		

		t = TileType(True)
		t.name = "NotEqual:{}".format(data)
		t.sides[tertiaryDirection] = Glue(1, "NotEqual")
		t.sides[secondaryDirection] = Glue(2, "NotEqual:{}".format(data))
		t.sides[secondaryDirectionReverse] = Glue(1, data)
		t.color = "Red"
		tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "NotEqual:{}:Prop".format(data)
		t.sides[primaryDirection] = Glue(1, "NotEqual:{}".format(data))
		t.sides[secondaryDirection] = Glue(1, data)
		t.sides[tertiaryDirection] = Glue(1, "NotEqual:Prop")
		tdsOut.AddTile(t)

	for role in ["Equal", "NotEqual"]:
		t = TileType(True)
		t.name = "{}First".format(role)
		t.sides[primaryDirectionReverse] = Glue(2, role)
		t.sides[tertiaryDirectionReverse] = Glue(1, role)
		t.color = roleColor(role)
		tdsOut.AddTile(t)

		for role2 in ["Equal", "NotEqual"]:
			t = TileType(True)
			t.name = "{}+{}".format(role, role2)
			t.sides[tertiaryDirection] = Glue(1, role)
			t.sides[primaryDirectionReverse] = Glue(1, role2)
			if role == "Equal":
				t.sides[tertiaryDirectionReverse] = Glue(1, role2)
				t.color = roleColor(role2)
			else:
				t.sides[tertiaryDirectionReverse] = Glue(1, role)
				t.color = roleColor(role)
			tdsOut.AddTile(t)

		for role2 in ["EqualDone", "NotEqualDone"]:
			t = TileType(True)
			result = ""
			t.sides[primaryDirectionReverse] = Glue(1, role2)
			t.sides[tertiaryDirection] = Glue(1, role)
			if role == "Equal":
				result = role2.replace("Done","")
				t.sides[secondaryDirection] = Glue(2, "Final:{}".format(result))
			else:
				result = role
				t.sides[secondaryDirection] = Glue(2, "Final:{}".format(result))
			t.name = "Final:{}+{}={}".format(role, role2, result)
			t.color = roleColor(result)
			tdsOut.AddTile(t)

		t = TileType(True)
		t.name = "Final:{}".format(role)
		t.sides[secondaryDirectionReverse] = Glue(2, "Final:{}".format(role))
		t.sides[primaryDirectionReverse] = Glue(1, role)
		t.color = roleColor(role)
		tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "EqualDone:0or1"
	t.sides[secondaryDirectionReverse] = Glue(2, "EqualDone:lm")
	t.sides[primaryDirectionReverse] = Glue(1, "Query")
	t.color = "Green"
	tdsOut.AddTile(t)

	t = TileType(True)
	t.name = "NotEqualDone:1lm"
	t.sides[secondaryDirectionReverse] = Glue(2, "NotEqualDone:lm")
	t.sides[secondaryDirection] = Glue(1, "1lm")
	t.sides[primaryDirectionReverse] = Glue(1, "Increment")
	t.color = "Red"
	tdsOut.AddTile(t)



	for side in DirAbbrevArray:
		t = TileType(True)
		t.name = "StartSideQuery:{}".format(side)
		t.sides[primaryDirection] = Glue(1, "Query")
		t.sides[tertiaryDirection] = Glue(1, "TD:{}".format(side+"Query"))
		t.sides[secondaryDirection] = Glue(1, "SideQuery:{}".format(side))
		t.color = "Cyan"
		tdsOut.AddTile(t)

		for role in baseTileNames:
			t = TileType(True)
			t.name = "{}:SideQuery:{}".format(role, side)
			t.sides[secondaryDirectionReverse] = Glue(1, "SideQuery:"+side)
			t.sides[tertiaryDirection] = Glue(1, role)
			if role == side:
				t.sides[secondaryDirection] = Glue(1, "Activate")
				t.color = "Cyan"
			else:
				t.sides[secondaryDirection] = Glue(1, "SideQuery:"+side)
				t.color = "Magenta"
			tdsOut.AddTile(t)

	for tile in baseTiles:
		t = TileType(True)
		t.name = "{}:PreInterface:GlueTable".format(tile.name)
		t.sides[secondaryDirection] = Glue(1, "Activate")
		t.sides[secondaryDirectionReverse] = Glue(1, "Activate")
		t.sides[tertiaryDirection] = Glue(1, tile.name)
		t.sides[tertiaryDirectionReverse] = Glue(2, "{}|Interface".format(tile.name))
		tdsOut.AddTile(t)

	tileClass = "InitializationKeyGen"
	t = TileType(True)
	t.name = tileClass
	t.sides[tertiaryDirection] = Glue(2, tileClass)
	t.sides[secondaryDirection] = Glue(1, "Activate")
	# t.sides[secondaryDirectionReverse] = Glue(1, "CallbackSimulation")
	tdsOut.AddTile(t)

	tileClass = "CallbackSimulation"
	t = TileType(True)
	t.name = tileClass
	t.sides[tertiaryDirection] = Glue(1, "CallbackHelper")
	t.sides[secondaryDirection] = Glue(1, tileClass)
	t.sides[secondaryDirectionReverse] = Glue(1, tileClass)
	tdsOut.AddTile(t)

	tileClass = "CallbackSimulationEnd"
	t = TileType(True)
	t.name = tileClass
	t.sides[tertiaryDirection] = Glue(1, "CallbackEnd")
	t.sides[secondaryDirection] = Glue(1, "CallbackSimulation")
	t.sides[secondaryDirectionReverse] = Glue(2, "CallbackTurn")
	tdsOut.AddTile(t)

	tileClass = "CallbackTurn"
	t = TileType(True)
	t.name = tileClass
	t.sides[tertiaryDirection] = Glue(1, tileClass)
	t.sides[secondaryDirection] = Glue(2, tileClass)
	tdsOut.AddTile(t)


	
	for tileName in initTileNamesActive:
		t = TileType(True)
		t.name = "{}:{}:PreInterface:Init".format(tileName, "Initialization")
		t.sides[secondaryDirection] = Glue(1, "Activate")
		t.sides[secondaryDirectionReverse] = Glue(1, "Activate")
		t.sides[tertiaryDirection] = Glue(1, ":".join([tileName, "Active"]))
		t.sides[tertiaryDirectionReverse] = Glue(2, "{}|Interface:Init".format(tileName))
		tdsOut.AddTile(t)

def InterfaceTiles():
	travelDir = "DN"
	dirs = PSTdir[travelDir]
	primaryDirection = DirAbbrevArray.index(dirs[0])
	secondaryDirection = DirAbbrevArray.index(dirs[1])
	tertiaryDirection = DirAbbrevArray.index(dirs[2])
	primaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[0]])
	secondaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[1]])
	tertiaryDirectionReverse = DirAbbrevArray.index(oppositeByDir[dirs[2]])

	for tile in tdsInData.tileList:
		if "Start" in tile.name and "DN" in tile.name:
			t = TileType(True)
			t.name = "{}|Interface".format(tile.name)
			t.sides[primaryDirectionReverse] = Glue(2, "{}|Interface".format(tile.name))
			t.sides[primaryDirection] = tile.sides[primaryDirection].Copy()
			t.sides[secondaryDirection] = Glue(1, "Callback")
			tdsOut.AddTile(t)

	for tile in tdsInInit.tileList:
		t = TileType(True)
		t.name = "{}|Interface:Init".format(tile.name)
		t.sides[primaryDirectionReverse] = Glue(2, "{}|Interface:Init".format(tile.name))
		t.sides[primaryDirection] = tile.sides[primaryDirection].Copy()
		t.sides[secondaryDirection] = Glue(1, "Callback")
		t.sides[secondaryDirectionReverse] = Glue(1, "ActivationCallback")
		tdsOut.AddTile(t)





turnTest = True
instructionTest = True

"""if not includeDataInSeed:
	genomeLength = SeedTest(instructionGroups, "EU", [0, 0, 0])
	SeedTest(instructionGroups, "WU", [genomeLength + 8, 0, 0])
else:
	genomeLength = SeedSimpleInstruction(instructionGroups, "EU", [0, 0, 0])
	SeedSimpleInstruction(instructionGroups, "WU", [genomeLength + 8, 0, 0])"""
#SeedTest(instructionGroups, "UN", [4, 0, 0])


#SeedSimpleInstruction(instructionGroups, "SU", [10, 0, 0])

#baseTileNames = []

directionsMap = {'N':N, 'S':S, 'E':E, 'W':W, 'U':U, 'D':D }

def generateAllGenomeTiles(inhibitList=[]):

	inhibit = [directionsMap[x] for x in inhibitList if x in directionsMap]

	generateTurnTiles()
	generateInstructionTiles()
	generateIntersectionTiles()
	generateExternalCommunicationTiles(inhibit=inhibit)
	generateQueryTiles()
	generateQueryCheckTiles()
	InterfaceTiles()

if len(argv) > 1:
	generateAllGenomeTiles(argv[1:])
else:
	generateAllGenomeTiles()

#tdsOut.Write("TDPTDS")
#tdsSeed.Write("TDPTDS")
#tdpOut.Write("TDPTDS")

print initTileNames

temp = []
for i, tile in enumerate(tdsOut.tileList):
	flag = False
	for bandMod in band1+band2+band0xband2:
		if (bandMod not in band0 or bandMod not in band1) and bandMod == tile.name.split(":")[-1]:
			flag = True
	if not flag:
		temp.append(tile)
# tdsOut.tileList = temp
tdsOut.Write("Components/TDS")
#tdsOut.Write("../Seeds/")