from TileType import *
from TDPUtilities import *
from TDSUtilities import *
from LocationUtilities import *


def isInt(data):
	try:
		int(data)
		return True
	except:
		return False

roles = ["rise", "fall", "nop", "rt", "lt"]
alias = {"rise":"Rise", "fall":"Fall", "rt":"RT", "lt":"LT", "nop":"Data"}

def PCN(l, index):
	result = (None,None,None)
	result[0] = l[index-1] if len(l)-1 > index-1 >= 0 else None
	result[1] = l[index] if len(l)-1 > index >= 0 else None
	result[2] = l[index+1] if len(l)-1 > index+1 >= 0 else None


def condenseInstructions(instructions):
	result = []

	condense = 0
	for i, instr in enumerate(instructions):
		if isInt(instr):
			condense += int(instr)
		elif instr in roles:
			if condense > 0:
				result.append(condense)
			elif condense < 0:
				raise ValueError("Negative distance in instructions\n{}\n{}".format(instructions, list(PCN(instructions, i))))
			result.append(instr)
			condense = 0
		else:
			raise ValueError("Unexpected: {}".format(instr))
	if condense:
		result.append(condense)

	return result	

#orig and dest are assumed to be in PST format
#def nextInstr(orig, dest):

def EvenOdd(n):
	result = []
	result.append(n/2)
	if(n%2):
		result.append("nop")
	return result

def normalizeInstructions(instructions):
	result = []
	adjustment = 0
	for instr in instructions:
		if isInt(instr):
			result += EvenOdd(int(instr)-3)
		elif instr in roles:
			result.append(instr)
	return result

def formatInstructions(instructions, payload="0", initialOrientation="NU", instructionLength=0):
	intermediate = ["RB", "Inst"]

	instructions = condenseInstructions(instructions)
	instructions = normalizeInstructions(instructions)

	for instr in instructions:
		if isInt(instr):
			if int(instr) < 0:
				continue
				#raise ValueError("Unexpected Value: {}\n{}".format(instr, instructions))
			intermediate.append("FW")
			for bit in format(instr, "b")[::-1]:
				intermediate.append("C{}".format(bit))
			intermediate.append("MSB")
		elif instr in roles:
			intermediate.append(alias[instr])
		else:
			raise ValueError("Unexpected Value: {}".format(instr))
	intermediate.append("Stop")
	while(len(intermediate) < instructionLength-1):
		intermediate.append("Data")
	intermediate.append("LB")

	if initialOrientation in ["DN"]:
		intermediate = intermediate[::-1]

	template = "{}-Start-{}-{}"
	result = []

	for i, role in enumerate(intermediate):
		if i < len(payload):
			result.append(template.format(role, initialOrientation, payload[i]))
		else:
			result.append(template.format(role, initialOrientation, "0"))	
	return result

def seedInstructions(tdp, tileList, payload, initialOrientation="NU", origin=[0,0,0]):
	
	for i, name in enumerate(tileList):
		seedLocation = AddLoc(origin, XYZ([0,-1*(i+1),0], initialOrientation))
		tdp.AddTile(name, seedLocation)



#This method adjusts the instruction sets of a group of datapaths aligned vertically
#such that the original distance between datapaths is maintained
#Does not work IF:
#	1. Datapaths do not begin with the same width
#		1.5 Datapaths EVER do not have the same width
#	2. The instructions contain a LEFT instruction (This causes some datapaths to change width, causing problem 1)
#	3. VerticalSeperation is odd.  NOPs are supposed to allow odd values, but this often causes problem 1.
def verticalHighway(baseInstructions, verticalSeperation, verticalPosition, numLanes):
	riseAlias = "rise"
	fallAlias = "fall"
	rightAlias = "rt"
	leftAlias = "lt"
	nopAlias = "???"

	maintainLaneInstructions = []
	result = []
	for i, instruction in enumerate(baseInstructions):
		if instruction == riseAlias:
			maintainLaneInstructions.append(verticalSeperation/2 * (numLanes-verticalPosition))
			maintainLaneInstructions.append(instruction)
			maintainLaneInstructions.append(verticalSeperation/2 * (numLanes-verticalPosition))
		elif instruction == fallAlias:
			maintainLaneInstructions.append(verticalSeperation/2 * (verticalPosition))
			maintainLaneInstructions.append(instruction)
			maintainLaneInstructions.append(verticalSeperation/2 * (verticalPosition))
		else:
			maintainLaneInstructions.append(instruction)

	result = condenseInstructions(maintainLaneInstructions)

	#print "{}/{}\n{}".format(verticalPosition, numLanes, maintainLaneInstructions)
	print result
	return result

def test():
	tdpOut = TDP("DataPathTest.tdp", "3D-alldirs.tds")

	instructions = [ 10, "fall", 15, "rise", 10, "rt", 15, "lt", 15]
	payload = "101110"
	initialOrientation = "NU"
	originPST = [0, 0, 10]
	verticalSeperation = 10
	payloads = ["100011", "100000", "111011"]
	for i, payload in enumerate(payloads):
		payloadInstructions = verticalHighway(instructions, verticalSeperation, i, len(payloads)-1)
		intermediate = formatInstructions(payloadInstructions, payload, initialOrientation)
		seedInstructions(tdpOut, intermediate, payload, initialOrientation, XYZ(origin, initialOrientation))
		origin = AddLoc(origin, XYZ([0,0,verticalSeperation], initialOrientation))

	tdpOut.Write("DatapathTest")

def test2():
	tdpOut = TDP("DataPathTest.tdp", "3D-alldirs.tds")

	origin=[0,0,0]
	payload = "101110"
	initialOrientation = "NU"
	instructions = [ 15, "fall", 15, "rise", 15, "rt", 15, "lt", 15]

	intermediate = formatInstructions(instructions, payload, initialOrientation)
	seedInstructions(tdpOut, intermediate, payload, initialOrientation, XYZ(origin, initialOrientation))

	tdpOut.Write("DatapathTest")


test2()