# This script generates a tdp that will create a datapath that will grow into the adder with the specified proportions

import sys
from itertools import izip

# pairwise iterator
def pairwise(iterable):
    a = iter(iterable)
    return izip(a, a)

# returns an array of tiles that represent a movement of n tiles forward
def forward(n):
    if 0 < n <= 3:
        return ['Data'] * n
    if n > 3:
        path = ['FW']
        d = int((n - 3) / 2) + 1
        bits = "{0:b}".format(d)
        for b in reversed(bits):
            path.append('C'+b)
        path.append('MSB')
        if n % 2 == 0:
            path.append('Data')
        return path
    else:
        return None

# generates placement instructions from row of pairs
def placement(name):
    return "Place-{" + name + "}"

# converts the isntruction and position to a tile name and position
def tile_name(inst):
    s = inst + "-Start-DN-0"
    return s

# generates the actual instruction tiles from the paths array
# note there will be a space between each one
def generate_paths(paths, padded):
    instructions = []
    tiles_list = []

    for path in paths:
        tiles = []
        tiles.append(tile_name('RB'))
        tiles.append(tile_name('Inst'))

        for inst in path:
           tiles.append(tile_name(inst))

        for i in range(padded - len(path) - 3):
            tiles.append(tile_name('Data'))

        tiles.append(tile_name('LBC'))
        tiles_list.append(tiles)

    for t in tiles_list:
        for i in reversed(t):
            instructions.append(i)
        instructions.append('Prop')

    # print instructions
    return instructions

def bracket_padded_length(m, T, tau):
    return 3 * m.bit_length() + T * (3 * (11 + 3 * (m + tau).bit_length()).bit_length())

# This function generates the instructions necessary for the initialization of the datapaths that will place the bracket
# =======================================================================================================
# = Parameters ==========================================================================================
# =======================================================================================================
# > bracket_size    : how many inputs will there be in the bracket. Will be rounded to next power of two
# > input_spacing   : how far apart are the inputs to the bracket 
# > distances       : represent how far down/north/west the path needs to move before it starts to place the bracket
def generate_bracket_initialization(paddedLength, scaleFactor, startingOffset, bracketSpacing, bracketCount, downDistance, northDistance, westDistance):

    paths = []

    # the current level of the bracket
    level = 0
    count = bracketCount / 2
    spacing = bracketSpacing

    downDistance = downDistance - 2
    northDistance = northDistance - paddedLength - 3
    westDistance = westDistance - 1 + startingOffset

    while count > 0:

        path = []
        path.extend(forward(downDistance))
        path.append('Rise')
        path.extend(forward(northDistance))
        path.append('LT')
        path.extend(forward(westDistance))

        for i in range(count):
            if not i == 0:
                path.extend(forward(spacing - 1))

            path.append(placement("Turn"))
            path.extend(forward(spacing / 2 - 7))
            
            path.extend(['Fall', 'Rise', 'Rise'])
            path.append(placement("Merge"))
            path.extend(['Data','Fall'])
            path.extend(forward(4))
            path.append('Fall')
            path.append(placement("Merge"))
            path.extend(['Data','Rise','Rise','Fall'])

            path.extend(forward(spacing / 2 - 8 + (spacing % 2)))
            path.append(placement("Turn"))

        paths.append(path)

        downDistance += 8
        westDistance += paddedLength + 1 + spacing / 2

        spacing *= 2
        count /= 2

    instructions = generate_paths(paths, paddedLength)
    return instructions
