# This script generates a tdp that will create a datapath that will grow into the adder with the specified proportions

import sys
from itertools import izip

# pairwise iterator
def pairwise(iterable):
    a = iter(iterable)
    return izip(a, a)

# returns an array of tiles that represent a movement of n tiles forward
def forward(n):
    if 0 < n <= 3:
        return ['Data'] * n
    if n > 3:
        path = ['FW']
        d = int((n - 3) / 2) + 1
        bits = "{0:b}".format(d)
        for b in reversed(bits):
            path.append('C'+b)
        path.append('MSB')
        if n % 2 == 0:
            path.append('Data')
        return path
    else:
        print "Attempted forward instruction using a negative number"
        return None

# converts the isntruction and position to a tile name and position
def tile_name(inst):
    s = inst + "-Start-DN-0"
    return s

# generates the actual instruction tiles from the paths array
# note there will be a space between each one
def generate_paths(paths, padded):
    instructions = []
    tiles_list = []

    for path in paths:
        tiles = []
        tiles.append(tile_name('RB'))
        tiles.append(tile_name('Inst'))

        for inst in path:
           tiles.append(tile_name(inst))

        for i in range(padded - len(path) - 3):
            tiles.append(tile_name('Data'))

        tiles.append(tile_name('LBC'))
        tiles_list.append(tiles)

    for t in tiles_list:
        for i in reversed(t):
            instructions.append(i)
        instructions.append('Prop')

    return instructions

def input_padded_length(m, T, tau):
    return max(25 + 8 * m.bit_length(), T.bit_length())

def generate_input_initialization(paddedLength, scaleFactor, startingOffset, downDistance, northDistance, westDistance):

    paths = []

    downDistance = downDistance - 1
    northDistance = northDistance - paddedLength - 2
    westDistance = westDistance - 1 + startingOffset

    indexOffset = [10, 8, 6, 4, 2, 0]
    downOffset = [15, 12, 9, 6, 3, 0]

    # U, W, N, D, S, E

    X = 1400
    UD = 120
    externalPaths = []
    externalPaths.append(forward(X - indexOffset[5]) + ['Fall', 'Data', 'Data', 'Fall'] + forward(scaleFactor) + ['Fall'] + forward(downOffset[5] + 140 - paddedLength) + ['LT'] + forward(120) + ['Rise'] + forward(X))
    externalPaths.append(forward(X - indexOffset[4]) + ['LT'] + forward(200) + ['Rise', 'Fall'] + forward(scaleFactor-200) + ['Rise'] + forward(140 - paddedLength) + ['LT'] + forward(X))
    externalPaths.append(forward(X - indexOffset[3]) + ['RT', 'Fall'] + forward(scaleFactor) + ['Fall'] + forward(120) + ['Rise', 'RT'] + forward(X))
    externalPaths.append(forward(X - indexOffset[2]) + ['RT'] + forward(scaleFactor-120) + ['RT', 'Rise'] + forward(140 - paddedLength) + ['LT', 'Fall'] + forward(X))
    externalPaths.append(forward(X - indexOffset[1]) + ['Rise'] + forward(120) + ['Fall'] + forward(scaleFactor) + ['RT', 'Fall', 'RT'] + forward(X))
    externalPaths.append(forward(X - indexOffset[0]) + ['Rise'] + forward(160) + ['Fall', 'Data', 'Data', 'Rise'] + forward(scaleFactor - paddedLength) + ['LT'] + forward(20) + ['Rise'] + forward(X))

    downDistance += 5 * 3
    westDistance += 5 * 2

    for i in range(6):

        path = []
        path.extend(forward(downDistance))
        path.append('Rise')
        path.extend(forward(northDistance))
        path.append('LT')
        path.extend(forward(westDistance))
        path.append('Var')
        # path.extend(['Data', 'Data', 'Data'])

        downDistance -= 3
        westDistance += paddedLength + 1 - 2

        path.extend(externalPaths[i])
        paths.append(path)

    downDistance += 7 * 3
    westDistance += 7 * 2

    path = []
    path.extend(forward(downDistance))
    path.append('Rise')
    path.extend(forward(northDistance))
    path.append('LT')
    path.extend(forward(westDistance))
    paths.append(path)

    return generate_paths(paths, paddedLength)

