from TileType import *
from Orientation import *
from LocationUtilities import *
from TDPUtilities import *
import math


################################### ROLE SETUP ##################################
#		These roles define what actions the genome can possibly take			#
#################################################################################

validRoles = ["Turn", "Initialize", "Intersection", "Prop"] + ["{}Query".format(d) for d in DirAbbrevArray]
singletonTiles = ["EndInstructionSet", "Interface1", "Interface2", "Interface3", "End", "StartPadding", "EndPadding"]

################################### INSTRUCTION SETUP ###################################
#		These Instructions tell the genome how far to move and what role to perform		#	
#		at each step																	#
#########################################################################################

band0 = ["NU", "SU", "ND", "SD", "DN", "UN", "DS", "US"]
band1 = ["EU", "WU", "ED", "WD", "DE", "UE", "DW", "UW"]
band2 = ["EN", "WN", "ES", "WS", "NE", "SE", "NW", "SW"]
band0xband1 = ["NU", "SU", "ND", "SD"]
band0xband2 = ["DN", "UN", "DS", "US"]
band1xband0 = ["EU", "WU", "ED", "WD"]
band1xband2 = ["DE", "UE", "DW", "UW"]
band2xband0 = ["EN", "WN", "ES", "WS"]
band2xband1 = ["NE", "SE", "NW", "SW"]

#crossBandSignals = {"NU":"NUxEU", "EU":"NUxEU", "ND":"NDxED", "ED":"NDxED",
#	"UN":"UNxEN", "EN":"UNxEN", "US":"USxES", "ES":"USxES",
#	"NE":"NExUE", "UE":"NExUE", "NW":"NWxUW", "UW":"NWxUW"}

crossBandSignals = {"NU":"NUxEU", "EU":"NUxEU", "ND":"NDxED", "ED":"NDxED",
	"NE":"NExUE", "UE":"NExUE", "NW":"NWxUW", "UW":"NWxUW"}

crossBandOffset = {k:(1 if (k.split("x")[0] in band1 or k.split("x")[1] in band1) else 2)
	for k in list(set(crossBandSignals.values()))}


def InstructionGroups(n, offset=2, genome=None):
	for k, v in crossBandOffset.items():
		crossBandOffset[k] = v*offset

	band0Length = n
	band1Length = n + (2 * offset - 1)
	band2Length = n + 2 * (2 * offset - 1)

	bandLength = [band0Length, band1Length, band2Length]
	print bandLength

	#queryGap = len(bin(len(genome)).replace("0b","")) + 3 if genome else 3
	queryGap = 3
	queryDirMod = "NU"
	initDirMod = "NU"

	instructionDir = {}
	for bandNum, band in enumerate([band0, band1, band2]):
		for dirMod in band:
			instructions = ["Intersection"]
			if dirMod == initDirMod:
				instructions += ["Initialize"] + ["Prop"] * queryGap
			if dirMod == queryDirMod:
				for i in range(6):
					instructions += ["{}Query".format(DirAbbrevArray[i])] + ["Prop"] * queryGap
			#instructions += ["Prop"] * (n - 1 + (bandNum * offset * 2) - len(instructions) - 3)
			instructions += ["Prop"] * (bandLength[bandNum] - len(instructions))
			instructions += ["Turn"]
			instructionDir[dirMod] = instructions

	groups = {}
	groupNum = 0
	for instructionList in instructionDir.values():
		if instructionList not in groups.values():
			groups["Group{}".format(groupNum)] = instructionList
			groupNum += 1
	
	groupNames = groups.keys()
	for dirMod, instructions in instructionDir.items():
		for k in groupNames:
			if instructions == groups[k]:
				groups[dirMod] = k
				break

	return groups

