from __future__ import print_function
from sys import argv

# returns the tiles necessary to move forward n tiles
def forwardCorrection(n):
    if 0 < n <= 3:
        return ['Data'] * n
    if n > 3:
        path = ['FW']
        d = int((n - 3) / 2) + 1
        bits = "{0:b}".format(d)
        for b in reversed(bits):
            path.append('C'+b)
        path.append('MSB')
        if n % 2 == 0:
            path.append('Data')
        return path
    else:
        return None

def finalizeInstructions(instructions, data):
    instList = []
    length = max(len(data), len(instructions))
    for i in range(length):
        inst = 'Data'
        if i < len(instructions):
            inst = instructions[i]
        bit = '0'
        if i < len(data):
            bit = data[i]

        inst += '-Start-NU-' + bit
        instList += [inst]
    return instList

def printInstructions(file, instructions, origin):
    file.write('datapath.tds\nTemperature=2\n')
    for inst in instructions:
        file.write('{} {} {} {}\n'.format(inst, origin[0], origin[1], origin[2]))
        origin[0] -= 1

def main():
    if len(argv) <= 2:
        print("please provide an instruction file and a tdp filename")

    origin = [0, 0, 0]
    data = ""
    instructions = ['RB', 'Inst']

    in_file = open(argv[1])
    out_file = open(argv[2], 'w')

    for line in in_file:
        token = line.strip().split()
        if len(token) >= 1:
            if token[0] == "origin":
                origin = [int(x) for x in token[1:]]
            elif token[0] == "data":
                data = token[1]
            elif token[0] == "FW":
                instructions += forwardCorrection(int(token[1]))
            elif token[0] in ["Rise", "Fall", "LT", "RT"]:
                instructions += [token[0]]
            else:
                print("unknown instruction", token[0])

    instructions += ['LBC']
    instructions = finalizeInstructions(instructions, data)
    printInstructions(out_file, instructions, origin)

    in_file.close()
    out_file.close()

if __name__ == "__main__":
    main()