from Components.TileType import *
from Components.TDPUtil import *
from Components.TDSUtil import *
from Components.TDPUtilities import *
from Components.TDSUtilities import *
from Components.genomeUtilities import InstructionGroups
from Components.LocationUtilities import *
from Components.Orientation import *
from Components.datapath import formatInstructions, condenseInstructions
from Components.Adder import adderTDP
from Components.GlueGenome import glueGenome, writeGenome
from Components.datapath_adder import generate_adder_initialization, adder_padded_length
from Components.datapath_bracket import generate_bracket_initialization, bracket_padded_length
from Components.datapath_input import generate_input_initialization, input_padded_length
from Components.scaleFactor import calcScaleFactor
import os
import sys
import copy
import math
from sys import argv
from Tkinter import Tk
from tkFileDialog import askopenfilename

preferredDirMod = {"N":"UN", "S":"US", "E":"UE", "W":"UW", "U":"NU", "D":"ND"}

movementSize         = 0
scaleFactor          = 0
adderPaddedLength    = 0
bracketPaddedLength  = 0
inputPaddedLength    = 0
simulatedTiles       = 0
simulatedTemperature = 0
diffLocation = None

def binary(n, temp):
	result = ""
	size = len(bin(temp).replace("0b",""))
	#print size
	result = format(n, "0{}b".format(size))[::-1]
	#print "Result = " + result
	return result

def chooseSimulation():
	Tk().withdraw()
	tdpPath = askopenfilename(initialdir="/Examples", title="Choose TDP file to simulate", filetypes=(("tdp files", "*.tdp"),))
	if not tdpPath:
		exit(-1)
	#tdpPath = "Simulate/CO-OP.tdp"
	tdpSim = ParseAssembly(tdpPath)
	tdpSim.name = os.path.split(tdpPath)[-1]

	tdsSim = tdpSim.tdsFile
	tdsSim = os.path.join(os.path.split(tdpPath)[0], tdsSim)
	tdsSim = ParseTileSet(tdsSim)

	return (tdpSim, tdsSim)

# same as chooseSimulation but using a command line filename instead of a file prompt
def chooseSimulationCL(tdpPath):
	tdpSim = ParseAssembly(tdpPath)
	if tdpSim is None:
		print "Unable to open TDP file: " + tdpPath
		exit(-1)

	tdpSim.name = os.path.split(tdpPath)[-1]

	tdsPath = os.path.join(os.path.split(tdpPath)[0], tdpSim.tdsFile)
	tdsSim = ParseTileSet(tdsPath)

	return (tdpSim, tdsSim)


########################################################################################################################
########################################################################################################################
########################################################################################################################

def calcMoveSize(genome=None, temp=2):
	return 2 * max([adderPaddedLength, bracketPaddedLength, inputPaddedLength])

def scaleFactorEstimate(T, tau):
	m = calcScaleFactor(T, tau)
	return m

def generateInstructions(minimumLength, genome=None):
	safetyFactor = 20
	result = instructionList(minimumLength, genome=genome)
	return result

def instructionList(n, offset=2, genome=None):
	result = []

	groups = InstructionGroups(n, offset, genome)

	groupNum = 0
	while(groups.get("Group{}".format(groupNum))):
		result.append("Group{}".format(groupNum))
		result += groups.get("Group{}".format(groupNum))
		groupNum += 1
	result.append("EndInstructionSet")
	return result

########################################################################################################################
########################################################################################################################
########################################################################################################################

def generateGenome(genome, temp, templateLength):
	iteration = 0
	maxLengthGuess = 0
	while(True):
		print "Iteration {}\tGuess={}".format(iteration, maxLengthGuess)
		success, result = genomeAttempt(genome, temp, templateLength, maxLengthGuess)
		if success:
			return result
		else:
			maxLengthGuess = result
			iteration += 1

def genomeAttempt(genome, temp, templateLength, maxLengthGuess):
	AdderHorizontalSpacing = maxLengthGuess
	AdderNorthDataVerticalOffset = 100
	AdderDataPathVerticalSeperation = 13

	result = []
	prevGlues = 0
	for tileNum, genomeSection in enumerate(genome):
		result += ["Check"]
		result += ["TD"]
		numCheckTiles = tileNum+1
		numTDTiles = tileNum+1
		for sideNum, genomeSnippet in enumerate(genomeSection):
			result += DirAbbrevArray[sideNum]
			numSides = 6*tileNum + sideNum + 1
			for targetNum, strength in enumerate(genomeSnippet):
				if strength == 0:
					continue
				payload = binary(strength, temp)
				#payload[0] += "m"
				#payload[-1] += "m"

				verticalTarget = (AdderNorthDataVerticalOffset - sideNum * (AdderDataPathVerticalSeperation))
				laneSeperation = ((6-sideNum) * len(genome))
				targetSeperation = targetNum

				returnToZero = (templateLength + numCheckTiles + numTDTiles + numSides + (prevGlues * maxLengthGuess) - maxLengthGuess)

				adderTarget = ((targetNum+1)*AdderHorizontalSpacing)

				initializationDatapathWidth = 80

				instructions = [verticalTarget + laneSeperation + targetSeperation]
				instructions += ["rise"]
				instructions += [(6-sideNum)*3]
				instructions += ["lt"]			
				instructions += [returnToZero]
				instructions += ["rt"]
				instructions += ["rt"]
				instructions += [adderTarget]
				instructions += ["lt"]
				instructions += [(6-sideNum)]
				instructions += ["rise"]
				instructions += [laneSeperation + targetSeperation]
				instructions += ["fall"]
				instructions += [15 + initializationDatapathWidth]

				instructions = condenseInstructions(instructions)
				formatted = formatInstructions(instructions, payload, "DN", maxLengthGuess)
				if (len(formatted) > maxLengthGuess):
					return (False, len(formatted))
				result += formatted
				prevGlues += 1

	tempTDP = TDP("AdderTemp.tdp", "AdderSeed.tds")
	for tileNum in range(len(genome)):
		AdderLocation = XYZ([3*maxLengthGuess + 12, (2*maxLengthGuess) + (tileNum*maxLengthGuess)-3, -1*AdderNorthDataVerticalOffset - 5 - 10], "NU")
		tempTDP.Merge(adderTDP(AdderLocation, temp))
	tempTDP.Write("temp")


	print "{} Glue Connections".format(prevGlues)
	print "{} Percent Glue Untilization".format("{0:.3g}".format(float(100*prevGlues) / (len(genome) * 6)))
	print "{} Percent Connectivity".format("{0:.3g}".format(float(100*prevGlues) / len(genome)**2 / 6))
	return (True, result)

########################################################################################################################
########################################################################################################################
########################################################################################################################

def calcInitializationLength():
	adderContrib = 19 * adderPaddedLength
	bracketContrib = simulatedTiles.bit_length() * (bracketPaddedLength + 1)
	inputContrib = 7 * (inputPaddedLength + 1)

	return adderContrib + bracketContrib + inputContrib

def nextPowerOf2(n):
    # make sure n is not a power of 2
    if n and not (n & (n - 1)):
        return n

    result = 1    
    while n > 0:
        n >>= 1
        result *= 2

    return result

#Wrapper function for adding Daniel's initialization tiles to the genome template
#Input Args:
#		--cumulativeGenome(int): number of tiles in genome so far
#		--adderPlacementTDP(TDP): TDP object containing expected tile placements for adders
#Returns:
#		--(List) List containing TILENAMES in the order they will be placed in the genome
def generateInitialization(cumulativeGenomeLength, adderPlacementTDP):
	startPositionPST = [0,cumulativeGenomeLength, 0]
	startPositionXYZ = XYZ(startPositionPST, "NU")

	# offset to account for initialization strip
	origin = startPositionXYZ
	origin[1] += 6
	origin[2] -= 3

	print "Datapath Origin =", origin

	print "\n=== CALCULATING ADDER SEED SPACING INFORMATION ===\n"
	# find spacing between adders
	# this will be spacing between the same tile of two adjacent adders

	# stores the x positions of the MainCounterSeed0 tiles
	counterSeedOffsets = []
	counterSeedLocations = []
	for loc, name in adderPlacementTDP.tiles.iteritems():
		if name == 'MainCounterSeed0':
			counterSeedLocations.append(loc)
			counterSeedOffsets.append(loc[0])

	counterSeedOffsets.sort()

	adderSpacing = abs(counterSeedOffsets[1] - counterSeedOffsets[0])
	adderCount = len(counterSeedOffsets)

	print "Number of Adders =", adderCount
	print "Adder Spacing =", adderSpacing

	# here we find the relative spacing among the rows of the adder
	row_coords = [set(), set(), set()]
	for loc, name in adderPlacementTDP.tiles.iteritems():
		if '0l|0l' in name or '1l|1l' in name:
			for i in [0, 1, 2]:
				row_coords[i].add(loc[i])

	rc_lists = [sorted(list(rc)) for rc in row_coords]
	
	# these represent the different offsets that occur between adjacent rows of the adder seed
	xSpacing = abs(rc_lists[0][1] - rc_lists[0][0])
	ySpacing = abs(rc_lists[1][1] - rc_lists[1][0])
	zSpacingMajor = abs(rc_lists[2][3] - rc_lists[2][2])
	zSpacingMinor = abs(rc_lists[2][1] - rc_lists[2][0])
	
	print "Adder Row Spacing:"
	print "\tX Offset =", xSpacing
	print "\tY Offset =", ySpacing
	print "\tZ Major Offset =", zSpacingMajor
	print "\tZ Minor Offset =", zSpacingMinor

	# here we find the tiles that make up the 3 primary rows of the adder seed
	
	# this tile is the tile with the smallest z, y, and x coordinates (with that order determining importance)
	base_tile = None
	for loc, name in adderPlacementTDP.tiles.iteritems():
		if base_tile is None:
			base_tile = loc
		else:
			if loc[2] < base_tile[2]:
				base_tile = loc
			elif loc[2] == base_tile[2]:
				if loc[1] < base_tile[1]:
					base_tile = loc
				elif loc[1] == base_tile[1]:
					if loc[0] < base_tile[0]:
						base_tile = loc

	print "Base Tile Location =", base_tile

	bracketInputOffset = counterSeedLocations[0][1] - base_tile[1]
	print "Bracket Input Offset =", bracketInputOffset

	rows = [[], [], []]
	base_loc = base_tile
	loc = base_loc

	for i in range(3):
		while loc in adderPlacementTDP.tiles:
			rows[i].append(adderPlacementTDP.tiles[loc])
			loc = (loc[0] + 1, loc[1], loc[2])
		base_loc = (base_loc[0], base_loc[1], base_loc[2] + zSpacingMinor)
		loc = base_loc

	print "Row Lengths:", len(rows[0]), len(rows[1]), len(rows[2])

	rowGrowTo = (base_tile[0] + 3 * adderSpacing + len(rows[0]), base_tile[1], base_tile[2])
	print "Base Row Grow To Location =", rowGrowTo

	# find the furthest MainCounterSeed0 tile
	furthest = None
	for loc in counterSeedLocations:
		if loc[0] == counterSeedOffsets[-1]:
			furthest = loc

	counterSeedGrowTo = (furthest[0] + 2, furthest[1], furthest[2])
	print "Counter Seed Grow To Location =", counterSeedGrowTo

	xCounterOffset = abs(counterSeedGrowTo[0] - rowGrowTo[0])
	yCounterOffset = abs(counterSeedGrowTo[1] - rowGrowTo[1])
	zCounterOffset = abs(counterSeedGrowTo[2] - rowGrowTo[2])

	print "Counter Seed Offsets =", xCounterOffset, yCounterOffset, zCounterOffset

	print "\n=== DONE CALCULATING ADDER SEED SPACING INFORMATION ===\n"

	# previously calculated padded lengths
	print "Adder Padded Length", adderPaddedLength
	print "Bracket Padded Length", bracketPaddedLength
	print "Input Padded Length", inputPaddedLength

	northDist = max([adderPaddedLength, bracketPaddedLength, inputPaddedLength]) + 5
	westDist = 20
	adderDistance   = [westDist, northDist, 110]

	westDist += len(rows[0]) + 2
	northDist += bracketInputOffset
	bracketDistance = [33, northDist, 120]

	# TODO: handle case with 1 tile
	westDist += ((simulatedTiles - 1) * adderSpacing) / ((simulatedTiles + 1).bit_length() - 1) + 1
	northDist += 1 + simulatedTiles.bit_length()
	inputDistance   = [westDist, northDist, 140]

	global diffLocation
	diffLocation = [origin[0] - inputDistance[0] - 2, origin[1] + inputDistance[1], origin[2] - inputDistance[2] + 1]

	tiles = []

	startingOffset = len(tiles)
	tiles += generate_input_initialization(inputPaddedLength, scaleFactor, startingOffset, inputDistance[2], inputDistance[1], inputDistance[0])

	startingOffset = len(tiles)
	tiles += generate_bracket_initialization(bracketPaddedLength, scaleFactor, startingOffset, adderSpacing, adderCount, bracketDistance[2], bracketDistance[1], bracketDistance[0])
	
	startingOffset = len(tiles)
	tiles += generate_adder_initialization(adderPaddedLength, scaleFactor, startingOffset, adderSpacing, adderCount, xSpacing, ySpacing, zSpacingMajor, zSpacingMinor, rows, xCounterOffset, yCounterOffset, adderDistance[2], adderDistance[1], adderDistance[0])

	return tiles

def SeedTileID(tdp, tileID):
	global simulatedTiles, diffLocation

	blockLocation = (diffLocation[0] - 5 * 2, diffLocation[1] - simulatedTiles.bit_length(), diffLocation[2] - 6 * 3)

	tdp.AddTile('Init-Carrier', blockLocation)

	bits = bin(tileID)[2:].rjust(simulatedTiles.bit_length(), '0')
	i = 0
	for bit in bits:
		if i == 0:
			suffix = 'm'
		elif i == simulatedTiles.bit_length() - 1:
			suffix = 'l'
		else:
			suffix = ''
		name = "Init-" + bit + suffix
		tdp.AddTile(name, tuple(diffLocation))
		diffLocation[1] -= 1
		i += 1
	tdp.AddTile("Init-Carrier", tuple(diffLocation))
	

########################################################################################################################
########################################################################################################################
########################################################################################################################

def generateDoneSignals():
	TODO = []
	dp_north = []
	dp_south = []
	dp_east = []
	dp_west = []
	dp_up = []
	dp_down = []

	return TODO

########################################################################################################################
########################################################################################################################
########################################################################################################################

def genomeTemplate(genome, temp):

	movementSize = calcMoveSize(genome=genome, temp=temp)

	template = ["Start", "StartPadding"]
	template += generateInstructions(movementSize, genome=genome)
	template += ["Interface1"] + ["Prop"] * len(bin(len(genome)).replace("0b",""))
	template += generateGenome(genome, temp, len(template))
	template += ["Interface2"]

	global scaleFactor
	scaleFactor = calcInitializationLength() + len(template) + 3 + 2 * movementSize

	template += generateInitialization(len(template), ParseAssembly("temp/AdderTemp.tdp"))
	template += ["Interface3"]
	template += ["EndPadding", "End"]
	return template

def genomeSeedActual(template):
	result = []
	for d in DirAbbrevArray:
		row = []
		for role in template:
			row.append(genomeSeedFormat(role, d))
		result.append(row)
	return result

def genomeSeedFormat(role, direction):
	startRole = Intersections["NU"]
	if role == "Start":
		role = startRole
	seedFormat = "{}:{}:{}".format(role, "Intersection", Intersections["NU"])
	return seedFormat

def queryTileNumSeedLocation(sideNum, genome=None, size=0):
	constant = 5

	querySide = 0
	tileCount = 0

	groups = InstructionGroups(size, offset=2, genome=genome)
	
	for instr in groups["Group3"]:
		tileCount += 1
		if "Query" in instr:
			if sideNum == querySide:
				break
			else:
				querySide += 1
	x = constant + tileCount
	y =  sum([len(g) for k, g in groups.items() if "Group" in k]) + 7
	z = -1
	return XYZ([x,y,z], "NU")

def seedQueryCheck(tdp, n, origin=[0,0,0]):
	binary = [bit for bit in bin(n).replace("0b","")[::-1]]

	if len(binary) == 1:
		binary[0] += "lm"
	elif len(binary) > 1:
		binary[0] += "l"
		binary[-1] += "m"
	#print "{}={}".format(n, binary)
	for i, data in enumerate(binary):
		tdp.AddTile("QueryCheckSeed:{}".format(data), AddLoc(origin, XYZ([0,0,-1*i],"NU")))

########################################################################################################################
########################################################################################################################
########################################################################################################################

def seedLocation(globalLoc):
	result = [0,0,0]
	baseLocation = [0,0,0]
	result[0] = baseLocation[0] + globalLoc[0] * scaleFactor
	result[1] = baseLocation[1] + globalLoc[1] * scaleFactor
	result[2] = baseLocation[2] + globalLoc[2] * scaleFactor
	return result

def seedGenome(tdpSeed, tdpSim, tdsSim, seedList):
	for loc, name in tdpSim.tiles.iteritems():
		tile = tdsSim.tiles[name]
		print loc
		for i, role in enumerate(seedList[0]):
			tdpSeed.AddTile(role, AddLoc(XYZ([0,i,0], "NU"), seedLocation(loc)))

		tileID = tdsSim.tiles.keys().index(name)
		SeedTileID(tdpSeed, tileID)


def main():
	if len(argv) >= 2:
		tdpPath = argv[1]
		tdpSim, tdsSim = chooseSimulationCL(tdpPath)
	else:
		tdpSim, tdsSim = chooseSimulation()

	name = tdpSim.name.replace(".tdp","")
	temp = tdpSim.temperature
	genome = glueGenome(tdsSim)
	writeGenome(name, temp, genome)
	
	global simulatedTiles, simulatedTemperature
	simulatedTiles = len(tdsSim.tiles)
	simulatedTemperature = temp

	print "NUMBER OF SIMULATED TILES", simulatedTiles
	print "TEMPERATURE OF SIMULATED SYSTEM", simulatedTemperature

	m = scaleFactorEstimate(simulatedTiles, simulatedTemperature)
	global adderPaddedLength, bracketPaddedLength, inputPaddedLength
	adderPaddedLength = adder_padded_length(m, simulatedTiles, simulatedTemperature)
	bracketPaddedLength = bracket_padded_length(m, simulatedTiles, simulatedTemperature)
	inputPaddedLength = input_padded_length(m, simulatedTiles, simulatedTemperature)

	genomeSeedTemplate = genomeTemplate(genome, temp)
	genomeSeedListByDir = genomeSeedActual(genomeSeedTemplate)

	IU_TDS_fileName = "Master.tds"
	tdpSeed = TDP("Simulate({})".format(name), IU_TDS_fileName)

	size = calcMoveSize(genome, temp)
	seedGenome(tdpSeed, tdpSim, tdsSim, genomeSeedListByDir)
	"""
	#Simulation(Axes)
	seedQueryCheck(tdpSeed, 2, queryTileNumSeedLocation(N, genome, size))
	seedQueryCheck(tdpSeed, 2, queryTileNumSeedLocation(S, genome, size))
	seedQueryCheck(tdpSeed, 3, queryTileNumSeedLocation(W, genome, size))
	seedQueryCheck(tdpSeed, 0, queryTileNumSeedLocation(D, genome, size))
	"""
	#Simulation(CO-OP)
	# seedQueryCheck(tdpSeed, 0, queryTileNumSeedLocation(N, genome, size))
	# seedQueryCheck(tdpSeed, 1, queryTileNumSeedLocation(E, genome, size))
	# seedQueryCheck(tdpSeed, 2, queryTileNumSeedLocation(S, genome, size))
	# seedQueryCheck(tdpSeed, 3, queryTileNumSeedLocation(W, genome, size))
	# seedQueryCheck(tdpSeed, 4, queryTileNumSeedLocation(U, genome, size))
	# seedQueryCheck(tdpSeed, 5, queryTileNumSeedLocation(D, genome, size))

	#tdpSeed.Merge(parseTDP("temp/AdderTemp.tdp"))
	tdpSeed.Write("Seeds")

	

main()